## BotFTP (c) 1999 by Amadeus - The Elite Scripting Team, efNET #scripting
set bftpver "\002BotFTP\002 v0.96 (c) 1999 Amadeus"
## BotFTP is based on ideas derived from sendftp proc (c) by Ernst
## BotFTP uses the bot shell's ftp client to provide ftp services to
## other scripts as well as to +n users in DCC chat.
##
## Features:
##      - supports put, get, rename, delete and mkdir
##      - supports multiple mirror sites
##      - encrypted mirror site config
##      - supports ports other that 21
##      - supports usernames with embedded spaces
##      - does not exit on exec ping or ftp failures, reports failures
##
## Script Usage:
##
##      Load this script before any others that use it. All calling 
##      scripts should do:
##          if {[info exists botftp_loaded]}
##      before calling and of these procs, to make sure that the BotFTP
##      functions are available.
##
##      Add the following site config encryption key to your bot config file, 
##      using some other value, of course:
##          set sitekey "bubbles0rt" (default if not set in bot config)
##
##      Mirror Sites Configuration
##  
##      .bf list                                  - decrypt and list sites
##      ,bf key <newkey>                          - change key temporarly
##      .bf add <ip> <port> <login> <pwd> <path>  - add a site
##      .bf del <ip>"                             - remove a site
##          ip      - ip or hostname of site
##          port    - ftp port
##          login   - user/login name
##          pwd     - password - use quotes if it contains spaces. 
##          path    - destination directory on remote site
##                  - use "/" if home dir of ftp
##                  - always use trailing slash
##
##      Mirror Sites Procedures:
##      The procs sendpage, sendpage_subdir, makepage_subdir, renamepage
##      will read the site config and access all listed sites.
##
##      proc sendpage { fullpathname }
##          sends fullpathname to configured path on each remote site.
##
##      proc sendpage_subdir { fullpathname }
##          will extract the LAST subdir in fullpath name and create
##          it on remote site, relative to the configured remote path.  
##          It will then send the file.
##
##      proc makepage_subdir { subdirname }
##          will create the subdir on all mirror sites, relative to the
##          configured remote path
##
##      proc deletepage { filename }
##          deletes file in the configured remote path, on all mirror sites
##
##      proc renamepage { oldname newname }
##          renames file in the configured remote path, on all mirror sites
##
##
##      FTP Direct Procedures:
##      The procs sendftp, getftp, mkdirftp and renameftp are usually
##      called by the Mirror Site procs, but may also be used to access
##      a single site. These procs do not use the mIrror Site config.
##
##      proc sendftp { localfile server port user pass remotefile }
##          sendftp was originally developed by ernst. This version contains 
##          extensive fixes and additional features and error handling.
##          localfile   - FULL pathname of the file to send on local machine
##          server      - ip or hostname of site
##          port        - ftp port
##          user        - user/login name
##          pass        - password - use quotes if it contains spaces. 
##          remotefile  - FULL path to the file to send on remote site
##
##      proc getftp { localpath server port user pass remotefile  } 
##          localpath   - FULL path of where you want file received on bot
##                      - you must have permissions
##          server      - ip or hostname of site
##          port        - ftp port
##          user        - user/login name
##          pass        - password - use quotes if it contains spaces. 
##          remotefile  - FULL path to the file to GET from remote site
##
##      proc mkdirftp { server port user pass newdir }
##          server      - ip or hostname of site
##          port        - ftp port
##          user        - user/login name
##          pass        - password - use quotes if it contains spaces. 
##          newdir      - fullpath of new directory
##
##      proc renameftp { oldfilename server port user pass newfilename }
##          oldfilename - fullpathname of old filename
##          server      - ip or hostname of site
##          port        - ftp port
##          user        - user/login name
##          pass        - password - use quotes if it contains spaces. 
##          newfilename - fullpathname of new filename
##
##      proc deleteftp { server port user pass fullpathname }
##          server      - ip or hostname of site
##          port        - ftp port
##          user        - user/login name
##          pass        - password - use quotes if it contains spaces. 
##          fullpathname - fullpathname of file to delete
##
##
## DCC Commands
##    .bf sendpage <fullpath>                   - send file to all sites
##    .bf sendpagedir <fullpath>                - send file to all sites
##                                              - creates subdir first
##    .bf makepagedir <newdir>                  - create subdir on all sites
##    .bf sendftp <localfile> <server> <port> <user> <pass> <remotefile>
##                                              - send file to specified site
##    .bf mkdirftp <server> <port> <user> <pass> <newdir>
##                                              - make directory on site
##    .bf delftp <server> <port> <user> <pass> <fullpathnamer>
##                                              - delete file from site
##    .bf renameftp <oldname> <server> <port> <user> <pass> <newname>
##                                              - rename a file on site
##    .bf getftp <localpath> <server> <port> <user> <pass> <remotefile>
##                                              - get a file from site
##
##
##  Known Problems:
##      - With a large number of mirror sites and/or files, the bot
##        will occasionally ping timeout while ftping.
##      - There is NO RETRY if, no ping response or ftp fails
##        My own scripts which use BotFTP (WebChanStats, ReleaseLists)
##        update sites every 30 mins. 
##  
##

## location of mirror sites configuration file
set sites_config "scripts/sites.config"

# ernst notes
# Change this to something to use to check if a host is alive.
# set pingcheck "" to disable this checking.
# "/bin/ping -c 1" works on Linux. Try just "/bin/ping" on other machines
# Set to "" to disable this checking
# Fix by Amadeus, use catch on ping to prevent error exit
set pingcheck "/bin/ping -c 1"

bind dcc n bf dcc:botftp
# main dcc binding
proc dcc:botftp {hand idx arg} {
  global websites sitekey bftpver
  set newarg [lrange $arg 1 end]
  switch [string tolower [lindex $arg 0]] {
    "add"           { set r [add_website $hand $idx $newarg] }
    "del"           { set r [del_website $hand $idx $newarg] }
    "sendpage"      { set r [sendpage $newarg] }
    "sendpagedir"   { set r [sendpage_subdir $newarg] }
    "makepagedir"   { set r [makepage_subdir $bewarg] }
    "list" {
      putdcc $idx "BotFTP: sitekey is $sitekey"
      foreach site $websites {
        set site [decrypt $sitekey $site]
        if {[llength $site]==5} {
          set webftp  [lindex $site 0]
          set webport [lindex $site 1]
          set webuser [lindex $site 2]
          set webpwd  [lindex $site 3]
          set webpath [lindex $site 4]
          putdcc $idx "BotFTP: $webftp $webport $webpath $webuser $webpwd"
        } else {
          putdcc $idx "BotFTP: Config error: $site"
        }
      }
      set r 1
    }
    "key" {
        if {$newarg == ""} {
          putdcc $idx "BotFTP: missing parameter"
          return 0
        }
        set newsitekey $newarg

        putdcc $idx "BotFTP: re-encryting from $sitekey to $newsitekey"
        set indx 0
        foreach site $websites {
          set site [decrypt $sitekey $site]
          set websites [lreplace $websites $indx $indx [encrypt $newsitekey $site]]
          incr indx
        }
        set sitekey $newsitekey
        putdcc $idx "BotFTP: re-encryption done, sitekey is now $sitekey"
        set r 1
    }
    "sendftp"  {
        if {[llength $newarg] < 6]} {
          putdcc $idx "BotFTP: insufficent arguments"
          return 0
        }
        set localfile [lindex $newarg 0]
        set server [lindex $newarg 1]
        set port [lindex $newarg 2]
        set user [lindex $newarg 3]
        set pass [lindex $newarg 4]
        set remotefile [lindex $newarg 5]
        set r [sendftp $localfile $server $port $user $pass $remotefile]
    }
    "getftp"    {
        if {[llength $newarg] < 6]} {
          putdcc $idx "BotFTP: insufficent arguments"
          return 0
        }
        set localpath [lindex $newarg 0]
        set server [lindex $newarg 1]
        set port [lindex $newarg 2]
        set user [lindex $newarg 3]
        set pass [lindex $newarg 4]
        set remotefile [lindex $newarg 5]
        set r [getftp $localpath $server $port $user $pass $remotefile]
    }
    "mkdirftp"  {
        if {[llength $newarg] < 5]} {
          putdcc $idx "BotFTP: insufficent arguments"
          return 0
        }
        set server [lindex $newarg 1]
        set port [lindex $newarg 2]
        set user [lindex $newarg 3]
        set pass [lindex $newarg 4]
        set newdir [lindex $newarg 5]
        set r [mkdirftp $server $port $user $pass $newdir]
    }
    "delftp"  {
        if {[llength $newarg] < 5]} {
          putdcc $idx "BotFTP: insufficent arguments"
          return 0
        }
        set server [lindex $newarg 1]
        set port [lindex $newarg 2]
        set user [lindex $newarg 3]
        set pass [lindex $newarg 4]
        set filename [lindex $newarg 5]
        set r [deleteftp $server $port $user $pass $filename]
    }
    "renameftp" {
        if {[llength $newarg] < 6]} {
          putdcc $idx "BotFTP: insufficent arguments"
          return 0
        }
        set oldfile [lindex $newarg 0]
        set server [lindex $newarg 1]
        set port [lindex $newarg 2]
        set user [lindex $newarg 3]
        set pass [lindex $newarg 4]
        set newfile [lindex $newarg 5]
        set r [renameftp $oldfile $server $port $user $pass $newfile]
    }
    default {
        putdcc $idx "$bftpver Command Help"
        putdcc $idx ".bf list       - list sites"
        putdcc $idx ".bf add <ip> <port> <login> <pwd> <path>"
        putdcc $idx ".bf del <ip>"
        putdcc $idx ".bf sendpage <fullpath>    - all sites"
        putdcc $idx ".bf sendpagedir <fullpath> - all sites, creates subdir first"
        putdcc $idx ".bf makepagedir <newdir>   - all sites, create subdir"
        putdcc $idx ".bf sendftp <localfile> <server> <port> <user> <pass> <remotefile>"
        putdcc $idx ".bf mkdirftp <server> <port> <user> <pass> <newdir>"
        putdcc $idx ".bf delftp <server> <port> <user> <pass> <fullpathname>"
        putdcc $idx ".bf renameftp <oldname> <server> <port> <user> <pass> <newname>"
        set r 0
    }
  }
  return $r
} 

proc read_siteconfig {} {
  global sites_config
  set res [ catch {
    set ldr [open $sites_config r]
    while {![eof $ldr]} {
      set line [string trim [gets $ldr]]
      if {$line!=""} { lappend websites $line }
    }
    close $ldr
  } ]
  if [info exists websites] { 
    return $websites
  } else {
    return ""
  }
}
proc write_siteconfig { websites } {
  global sites_config
  set res [ catch {
    set ldr [open $sites_config w]
    foreach line $websites {
      puts $ldr $line
    }
    close $ldr
  } ]
  if {$res} {
    return 0
  } else {
    return 1
  }
}

proc add_website {hand idx arg} {
  global websites sitekey
  if {[llength $arg]!=5} {
    putdcc $idx "BotFTP: insufficent arguments"
    return 0
  }
  set websites [read_siteconfig]
  set tmppat "[lindex $arg 0]*"
  foreach elm $websites {
    set tmpidx [lsearch [decrypt $sitekey $elm] $tmppat]
    if {$tmpidx != -1} {
      set websites [lreplace $websites $tmpidx $tmpidx "[encrypt $sitekey $arg]"]
      putdcc $idx "BotFTP: replaced $arg"
      return 1
    }
  }
  lappend websites "[encrypt $sitekey $arg]"
  putdcc $idx "BotFTP: appended $arg"
  write_siteconfig $websites
  return 1
}

proc del_website {hand idx arg} {
  global websites sitekey
  set tmppat "[lindex $arg 0]*"
  foreach elm $websites {
    set tmpidx [lsearch [decrypt $sitekey $elm] $tmppat]
    if {$tmpidx != -1} {
      set websites [lreplace $websites $tmpidx $tmpidx]
      putdcc $idx "BotFTP: $arg removed"
      return 1
    }
  }
  putdcc $idx "BotFTP: $arg not listed"
  write_siteconfig $websites
  return 0
} 


#
# Send file to sub-directory on a site
#
proc sendpage_subdir { fullpathpage } {
  global websites sitekey
  set dirs [file split [file dirname $fullpathpage]]
  set subdir [lindex $dirs end]
  set file "$subdir/[file tail $fullpathpage]"
  
  foreach site $websites {
    set wsite [decrypt $sitekey $site]
    if {[llength $wsite]==5} {
      set webftp  [lindex $wsite 0]
      set webport [lindex $wsite 1]
      set webuser [lindex $wsite 2]
      set webpwd  [lindex $wsite 3]
      set webpath [lindex $wsite 4]
      set ftpdest "$webpath$file"
      putlog "BotFTP: Creating $webpath/$subdir on $webftp"
      if {![mkdirftp $webftp $webport $webuser $webpwd $subdir]} {
        putlog "BotFTP: failed makedir, unable to send $ftpdest"
      }

      putlog "BotFTP: Sending $fullpathpage to $webftp$webpath$file"
      putlog "BotFTP: [sendftp $fullpathpage $webftp $webport $webuser $webpwd $ftpdest]"
    } else {
      putlog "BotFTP: Config error: [lindex $wsite 0]"
    }
  }
}
proc makepage_subdir { newdir } {
  global websites sitekey
  foreach site $websites {
    set wsite [decrypt $sitekey $site]
    if {[llength $wsite]==5} {
      set webftp  [lindex $wsite 0]
      set webport [lindex $wsite 1]
      set webuser [lindex $wsite 2]
      set webpwd  [lindex $wsite 3]
      set webpath [lindex $wsite 4]
      putlog "BotFTP: Creating $webpath/$subdir on $webftp"
      mkdirftp $webftp $webport $webuser $webpwd $subdir
    }
  }
}

proc sendpage { fullpathpage } {
  global websites sitekey
  set file [file tail $fullpathpage]
  
  foreach site $websites {
    set wsite [decrypt $sitekey $site]
    if {[llength $wsite]==5} {
      set webftp  [lindex $wsite 0]
      set webport [lindex $wsite 1]
      set webuser [lindex $wsite 2]
      set webpwd  [lindex $wsite 3]
      set webpath [lindex $wsite 4]
      set ftpdest "$webpath$file"

      putlog "BotFTP: Sending $fullpathpage to $webftp$webpath$file"
      putlog "BotFTP: [sendftp $fullpathpage $webftp $webport $webuser $webpwd $ftpdest]"
    } else {
      putlog "BotFTP: Config error: [lindex $wsite 0]"
    }
  }
}

proc renamepage { old new } {
  global websites sitekey
  foreach site $websites {
    set wsite [decrypt $sitekey $site]
    if {[llength $wsite]==5} {
      set webftp  [lindex $wsite 0]
      set webport [lindex $wsite 1]
      set webuser [lindex $wsite 2]
      set webpwd  [lindex $wsite 3]
      set webpath [lindex $wsite 4]
      putlog "BotFTP: Renaming $webpath$old to $webpath$new on $webftp"
      putlog "BotFTP: [renameftp $webpath$old $webftp $webport $webuser $webpwd $webpath$new]"
    } else {
      putlog "BotFTP: Config error: [lindex $wsite 0]"
    }
  }
}

proc deletepage { filename } {
  global websites sitekey
  foreach site $websites {
    set wsite [decrypt $sitekey $site]
    if {[llength $wsite]==5} {
      set webftp  [lindex $wsite 0]
      set webport [lindex $wsite 1]
      set webuser [lindex $wsite 2]
      set webpwd  [lindex $wsite 3]
      set webpath [lindex $wsite 4]
      putlog "BotFTP: Deleting $webpath$filename on $webftp"
      putlog "BotFTP: [deleteftp $webftp $webport $webuser $webpwd $webpath$filename]"
    } else {
      putlog "BotFTP: Config error: [lindex $wsite 0]"
    }
  }
}
# make directory on ftpsite
# directory name is relative
proc mkdirftp { server port user pass newdir } {
  global pingcheck
  set noftp [catch {set ftpprog [exec which ftp]}] 
  if {$noftp} {
    if {[file executable /usr/bin/ftp]} {
      set ftpprog /usr/bin/ftp
      set noftp 0
    }
    if {[file executable /bin/ftp]} {
      set ftpprog /bin/ftp
      set noftp 0
    }
  }
  if {$noftp} {
    return "mkdirftp You don't seem to have the 'ftp' tool"
  }
  set res [ catch {
    set pipe [open "|$ftpprog -n $server $port" w]
    if {[llength $user]>1} {
      puts $pipe "user \"$user\" $pass"
    } else {
      puts $pipe "user $user $pass"
    }
    puts $pipe "bin"
    puts $pipe "mkdir $newdir"
    puts $pipe "quit"
    close $pipe
  }]
  if {$res} {
    putlog "BotFTP: error with $server $res"
    return 0
  } else {
    return 1
  }
}

proc renameftp { oldfilename server port user pass newfilename } {
  global pingcheck
  set noftp [catch {set ftpprog [exec which ftp]}] 
  if {$noftp} {
    if {[file executable /usr/bin/ftp]} {
      set ftpprog /usr/bin/ftp
      set noftp 0
    }
    if {[file executable /bin/ftp]} {
      set ftpprog /bin/ftp
      set noftp 0
    }
  }
  if {$noftp} {
    return " Rename You don't seem to have the 'ftp' tool"
  }
  set res [ catch {
    set pipe [open "|$ftpprog -n $server $port" w]
    if {[llength $user]>1} {
      puts $pipe "user \"$user\" $pass"
    } else {
      puts $pipe "user $user $pass"
    }
    puts $pipe "bin"
    puts $pipe "delete $newfilename"
    puts $pipe "rename $oldfilename $newfilename"
    puts $pipe "quit"
    close $pipe
  }]
  if {$res} {
    putlog "BotFTP: error with $server $res"
    return 0
  } else {
    return 1
  }
}
#
proc deleteftp { server port user pass fullpathname } {
  global pingcheck
  set noftp [catch {set ftpprog [exec which ftp]}] 
  if {$noftp} {
    if {[file executable /usr/bin/ftp]} {
      set ftpprog /usr/bin/ftp
      set noftp 0
    }
    if {[file executable /bin/ftp]} {
      set ftpprog /bin/ftp
      set noftp 0
    }
  }
  if {$noftp} {
    return " Delete You don't seem to have the 'ftp' tool"
  }
  set res [ catch {
    set pipe [open "|$ftpprog -n $server $port" w]
    if {[llength $user]>1} {
      puts $pipe "user \"$user\" $pass"
    } else {
      puts $pipe "user $user $pass"
    }
    puts $pipe "bin"
    puts $pipe "delete $fullpathname"
    puts $pipe "quit"
    close $pipe
  }]
  if {$res} {
    putlog "BotFTP: error with $server $res"
    return 0
  } else {
    return 1
  }
}

# sendftp by ernst (fixes and additional features by Amadeus)
# Call it with all parameters given.  'localfile' and 'remotefile' *must* both
# be given as FULL paths to the filenames, the first on the local, the second
# on the remote  server.  For example:
# sendftp /home/bill/stats.htm www.gov.org 21 bill secret /home/bgates/public_html/stats.html
## sendftp fixes by Amadeus
## - added port
## - was typo 'ftd' instead of ftp
## - was error setting ftpprog to /usr/bin/ftp when /bin/ftp was found
## - added catch on pexec ping to prevent error exit
##
proc sendftp { localfile server port user pass remotefile } {
  global pingcheck
  if {![file exist $localfile]} {
    return "BotFTP: \[sendftp\] File $localfile does not exist."
  }
  if {$pingcheck != ""} {
    if {[catch {exec [lindex $pingcheck 0] [lrange $pingcheck 1 end] $server > /dev/null 2> /dev/null}]} {
      return "BotFTP: \[sendftp\] Machine $server does not seem to be alive."
    }
  }
  set noftp [catch {set ftpprog [exec which ftp]}] 
  if {$noftp} {
    if {[file executable /usr/bin/ftp]} {
      set ftpprog /usr/bin/ftp
      set noftp 0
    }
    if {[file executable /bin/ftp]} {
      set ftpprog /bin/ftp
      set noftp 0
    }
  }
  if {$noftp} {
    return "sendftp: You don't seem to have the 'ftp' tool"
  }

  set res [catch {
    set pipe [open "|$ftpprog -n $server $port" w]
    if {[llength $user]>1} {
      puts $pipe "user \"$user\" $pass"
    } else {
      puts $pipe "user $user $pass"
    }
    puts $pipe "bin"
    puts $pipe "put $localfile $remotefile"
    puts $pipe "quit"
    close $pipe
  }]
  if {$res} {
    putlog "BotFTP: error with $server $res"
    return 0
  } else {
    return 1
  }
}

proc getftp { localpath server port user pass remotefile  } {
  global pingcheck
  if {$pingcheck != ""} {
    if {[catch {exec [lindex $pingcheck 0] [lrange $pingcheck 1 end] $server > /dev/null 2> /dev/null}]} {
      return "BotFTP: \[sendftp\] Machine $server does not seem to be alive."
    }
  }
  set noftp [catch {set ftpprog [exec which ftp]}] 
  if {$noftp} {
    if {[file executable /usr/bin/ftp]} {
      set ftpprog /usr/bin/ftp
      set noftp 0
    }
    if {[file executable /bin/ftp]} {
      set ftpprog /bin/ftp
      set noftp 0
    }
  }
  if {$noftp} {
    return "sendftp: You don't seem to have the 'ftp' tool"
  }
  set remotepath [file dirname $remotefile]
  set remotefn [file tail $remotefile]
  set localfile $localpath$remotefn

  set res [catch {
    set pipe [open "|$ftpprog -n $server $port" w]
    if {[llength $user]>1} {
      puts $pipe "user \"$user\" $pass"
    } else {
      puts $pipe "user $user $pass"
    }
    puts $pipe "lcd $localpath"
    puts $pipe "bin"
    puts $pipe "cd $remotepath"
    puts $pipe "get $remotefn"
    puts $pipe "quit"
    close $pipe
  }]
  if {$res} {
    putlog "BotFTP: error with $server $res"
    return 0
  } else {
    if {[file isfile $localfile]} {
      putlog "BotFTP: received $localfile"
    }
    if {[file isfile $localfile]} {
      putlog "BotFTP: received $localfile"
      return 1
    } else {
      putlog "BotFTP: cant find $localfile"
      return 0
    }
  }
}

## blowfish key
if {![info exists sitekey]} { set sitekey $owner }
set websites [read_siteconfig]

set botftp_loaded 1
putlog "$bftpver loaded"
