#
#   tcl ftp package -- 
# 
#   Required:   tcl8.0b1
#
#   Created:	12/96 
#   Changed:    08/97                            
#   Version:    0.81
#
#   core ftp support: 	ftp::Open <server> <user> <passwd>
#		     	ftp::Close
#		    	ftp::Cd <directory>
#			ftp::Pwd
#			ftp::Type <ascii|binary>	
#			ftp::List <directory>
#			ftp::NList <directory>
#			ftp::Delete <file>
#			ftp::Rename <from> <to>
#			ftp::Put <local> <?remote?>
#			ftp::Get <remote> <?local?>
#			ftp::MkDir <directory>
#			ftp::RmDir <directory>
#
#   Copyright (C) 1997, Steffen Traeger
#	EMAIL:	Steffen.Traeger@t-online.de
#	URL:	http://home.t-online.de/home/Steffen.Traeger
#
#   This package is free software; you can redistribute it and/or 
#   modify it. 
#   This library is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
#   The description of the procedures are partly exerpted from RFC 959.
#   For more details see also RFC 959.
#
########################################################################

package provide FTP [lindex {$Revision: 0.8 $} 1]

namespace eval FTP {

namespace export DisplayMsg Open Close Cd Pwd Type List NList \
		 Delete Rename Put Get MkDir RmDir
	
	
#############################################################################
#
# DisplayMsg --
#
# This is a simple procedure to display any messages on screen.
# It must be overwritten by users source code in the form:
# (exported)
#
#	namespace ftp {
#		proc DisplayMsg {msg} {
#			......
#		}
#	}
#
# Arguments:
# msg - 		message string

proc DisplayMsg {msg} {



}

#############################################################################
#
# StateHandler --
#
# Implements a finite state handler and a fileevent handler
# for the control channel
# 
# Arguments:
# sock - 		socket name
#			If called from a procedure than this argument is empty.
# 			If called from a fileevent than this argument contains
#			the socket channel identifier.

proc StateHandler {{sock ""}} {
variable ftp 

	# there is no socket (and no channel to get) if called from a procedure
	set rc "   "
	if { $sock != "" } {
		set number [gets $sock buffer]

		if { $number > 0 } {

			# get return code, check for multi-line text
			regexp "(^\[0-9\]+)( |-)?(.*)\r?" $buffer all rc multi_line

			DisplayMsg "C: $buffer" 

		    if { $ftp(State) == "site_sent" } {
			lappend ftp(Site) [string range $buffer 4 end]
		    }

		    if { $ftp(State) == "quote_sent" || $ftp(State) == "quotew_sent" } {
			lappend ftp(Quote) [string range $buffer 4 end]
		    }
		

			# multi-line format detected ("-"), get all the lines
			# until the real return code
			while { $multi_line == "-"  } {
				set number [gets $sock buffer]	
				if { $number > 0 } {
					regexp "^\[0-9\]+" $buffer rc
					DisplayMsg "C: $buffer"
				        if { $ftp(State) == "site_sent" } {
					    lappend ftp(Site) [string range $buffer 4 end]
				        }
				        if { $ftp(State) == "quote_sent" || $ftp(State) == "quotew_sent" } {
					    lappend ftp(Quote) [string range $buffer 4 end]
				        }
 					regexp "(^\[0-9\]+)( |-)?(.*)\r?" $buffer all rc multi_line
				}
			}
		}
	} 
	
	if { [info exist ftp(Debug)] && $ftp(Debug)} {
		DisplayMsg "DEBUG: rc=$rc state=$ftp(State)"
	}
		
	switch -- $ftp(State) {
	
		user	{ 
			  switch $rc {
			    220     {
			    	       puts $ftp(CtrlSock) "USER $ftp(User)"
			               set ftp(State) passwd
			            }
			    default {
				       DisplayMsg "ERROR: Error connecting!"
				       uplevel #0 "set login 0"
			            }
			  }
			}

		passwd	{
			  switch $rc {
			    230     {
			               uplevel #0 "set login 1"
			            }
			    331     {
			  	       puts $ftp(CtrlSock) "PASS $ftp(Passwd)"
			  	       set ftp(State) connect
			            }
			    default {
				       DisplayMsg "ERROR: Error connecting!"
				       uplevel #0 "set login 0"
			            }
			  }
			}

		connect	{
			  switch $rc {
			    230     {
			               uplevel #0 "set login 1"
			  	       set ftp(State) noop
			            }
			    default {
				       DisplayMsg "ERROR: Error connecting!"
				       uplevel #0 "set login 0"
			            }
			  }
			}

		quit	{
		    	   puts $ftp(CtrlSock) "QUIT"
			   set ftp(State) quit_sent
			}

		quit_sent {
			  switch $rc {
			    221     {
			               uplevel #0 "set closed 1"
			            }
			    default {
				       DisplayMsg "ERROR: Error disconnecting!"
				       uplevel #0 "set closed 0"
			            }
			  }
			}
		
		abort	{
		    	   puts $ftp(CtrlSock) "ABOR"
			   set ftp(State) abort_sent
			}

		abort_sent {
			  switch $rc {
			    225     {
			               uplevel #0 "set aborted 1"
			               uplevel #0 "set closed 0"
			            }
			    default {
				       DisplayMsg "ERROR: Error aborting!"
				       uplevel #0 "set aborted 0"
			            }
			  }
			}
		
		type	{
		  	  if { $ftp(Type) == "ascii" } {
			  	puts $ftp(CtrlSock) "TYPE A"
			  } else {
			  	puts $ftp(CtrlSock) "TYPE I"
			  }
  		  	  set ftp(State) type_sent
			}
			
		type_sent {
			  switch $rc {
			    200     {
			               uplevel #0 "set closed 0"
			  	       set ftp(State) noop
			            }
			    default {
				       DisplayMsg "ERROR:  Error setting type \"$ftp(Type)\"!"
				       uplevel #0 "set closed 0"
			            }
			  }
			}
	
		nlist	{
			  if [OpenDataConn] {
			    	puts $ftp(CtrlSock) "PORT $ftp(LocalAddr),$ftp(DataPort)"
			  	set ftp(State) nlist_open
			  } else {
				DisplayMsg "ERROR: Error setting port!"
			  }
			  
			}
			
		nlist_open {
			  switch $rc {
			    200     {
			               puts $ftp(CtrlSock) "NLST$ftp(Dir)"
			  	       set ftp(State) list_sent
			            }
			    default {
				       DisplayMsg "ERROR: Error setting port!"
				       uplevel #0 "set closed 0"
			            }
			  }
			}
	
		list	{
			  if [OpenDataConn] {
			    	puts $ftp(CtrlSock) "PORT $ftp(LocalAddr),$ftp(DataPort)"
			  	set ftp(State) list_open
			  } else {
				DisplayMsg "ERROR: Error setting port!"
			  }
			  
			}
			
		list_open {
			  switch $rc {
			    200     {
			               puts $ftp(CtrlSock) "LIST$ftp(Dir)"
			  	       set ftp(State) list_sent
			            }
			    default {
				       DisplayMsg "ERROR: Error setting port!"
				       uplevel #0 "set closed 0"
			            }
			  }
			}
			
		list_sent	{
			  switch $rc {
			    150     {
			               set ftp(State) list_close
			            }
			    default {
				       DisplayMsg "ERROR: Error opening connection!"
				       uplevel #0 "set closed 0"
			            }
			  }
			}

		list_close {
			  switch $rc {
			    226     {
			               uplevel #0 "set closed 1"
			               set ftp(State) noop
			            }
			    default {
				       DisplayMsg "ERROR: Error receiving list!"
				       uplevel #0 "set closed 0"
			            }
			  }
			}


		site    {
		           puts $ftp(CtrlSock) "SITE $ftp(Cmd)"
		           set ftp(State) site_sent
		        }

		site_sent {
                                       uplevel #0 "set closed 0"
                                       set ftp(State) noop
		}

		quote   {
		           puts $ftp(CtrlSock) "$ftp(Cmd)"
		           set ftp(State) quote_sent
		        }

		quote_sent {
                                       uplevel #0 "set closed 0"
                                       set ftp(State) noop
		}

		quotew   {
		           puts $ftp(CtrlSock) "$ftp(Cmd)"
		           set ftp(State) quotew_sent
		        }

		quotew_sent {
			switch $rc {
				226 {
                                       uplevel #0 "set closed 0"
                                       set ftp(State) noop
				    }
			}
		}
    

		pwd	{
			   puts $ftp(CtrlSock) "PWD"
  		  	   set ftp(State) pwd_sent
			}
			
		pwd_sent {
			  switch $rc {
			    257     {
            			       regexp "^.*\"(.*)\"" $buffer temp ftp(Dir)
			               uplevel #0 "set closed 0"
			               set ftp(State) noop
			            }
			    default {
				       DisplayMsg "ERROR: Error getting working dir!"
				       uplevel #0 "set closed 0"
			            }
			  }
			}

		cd	{
			   puts $ftp(CtrlSock) "CWD $ftp(Dir)"
  		  	   set ftp(State) cd_sent
			}
			
		cd_sent {
			  switch $rc {
			    250     {
			               uplevel #0 "set closed 0"
			               set ftp(State) noop
			            }
			    default {
				       DisplayMsg "ERROR: Error changing to \"$ftp(Dir)\"!"
				       uplevel #0 "set closed 0"
				     }
			  }
			}
			
		mkdir	{
			  puts $ftp(CtrlSock) "MKD $ftp(Dir)"
  		  	  set ftp(State) mkdir_sent
			}
			
		mkdir_sent {
			  switch $rc {
			    257     {
			               uplevel #0 "set closed 0"
			               set ftp(State) noop
			            }
			    default {
				       DisplayMsg "ERROR: Error making dir \"$ftp(Dir)\"!"
				       uplevel #0 "set closed 0"
				     }
			  }
			}
			
		rmdir	{
			  puts $ftp(CtrlSock) "RMD $ftp(Dir)"
  		  	  set ftp(State) rmdir_sent
			}
			
		rmdir_sent {
			  switch $rc {
			    250     {
			               uplevel #0 "set closed 0"
			               set ftp(State) noop
			            }
			    default {
				       DisplayMsg "ERROR: Error removing dir \"$ftp(Dir)\"!"
				       uplevel #0 "set closed 0"
				     }
			  }
			}
										
		delete	{
			  puts $ftp(CtrlSock) "DELE $ftp(File)"
  		  	  set ftp(State) delete_sent
			}
			
		delete_sent {
			  switch $rc {
			    250     {
			               uplevel #0 "set closed 0"
			               set ftp(State) noop
			            }
			    default {
				       DisplayMsg "ERROR: Error deleting file \"$ftp(File)\"!"
				       uplevel #0 "set closed 0"
				     }
			  }
			}
			
		rename	{
			  puts $ftp(CtrlSock) "RNFR $ftp(RenameFrom)"
  		  	  set ftp(State) rename_to
			}
			
		rename_to {
			  switch $rc {
			    350     {
			  	       puts $ftp(CtrlSock) "RNTO $ftp(RenameTo)"
  		  	  	       set ftp(State) rename_sent
			            }
			    default {
				       DisplayMsg "ERROR: Error renaming file \"$ftp(RenameFrom)\"!"
				       uplevel #0 "set closed 0"
				     }
			  }
			}

		rename_sent {
			  switch $rc {
			    250     {
			               uplevel #0 "set closed 0"
			               set ftp(State) noop
			            }
			    default {
				       DisplayMsg "ERROR: Error renaming file \"$ftp(RenameFrom)\"!"
				       uplevel #0 "set closed 0"
				     }
			  }
			}
			
		put 	{
			   OpenDataConn
			   puts $ftp(CtrlSock) "PORT $ftp(LocalAddr),$ftp(DataPort)"
			   set ftp(State) put_open
			}
			
		put_open {
			  switch $rc {
			    200     {
			               puts $ftp(CtrlSock) "STOR $ftp(RemoteFilename)"
			               set ftp(State) put_sent
			            }
			    default {
				       DisplayMsg "ERROR: Error setting port!"
				       uplevel #0 "set closed 0"
				     }
			  }
			}
			
		put_sent	{
			  switch $rc {
			    150     {
			               set ftp(State) put_close
			            }
			    default {
				       DisplayMsg "ERROR: Error opening connection!"
				       uplevel #0 "set closed 0"
				     }
			  }
			}
			
		put_close	{
			  switch $rc {
			    226     {
			  	       uplevel #0 "set closed 1"
			               set ftp(State) noop
			            }
			    default {
				       DisplayMsg "ERROR: Error storing file \"$ftp(RemoteFilename)\"!"
				       uplevel #0 "set closed 0"
				     }
			  }
			}
			
		get 	{
			   OpenDataConn
			   puts $ftp(CtrlSock) "PORT $ftp(LocalAddr),$ftp(DataPort)"
			   set ftp(State) get_open
			}
			
		get_open {
			  switch $rc {
			    200     {
			               puts $ftp(CtrlSock) "RETR $ftp(RemoteFilename)"
			               set ftp(State) get_sent
			            }
			    default {
				       DisplayMsg "ERROR: Error setting port!"
				       uplevel #0 "set closed 0"
				     }
			  }
			}
			
		get_sent	{
			  switch $rc {
			    150     {
			               set ftp(State) get_close
			            }
			    default {
				       DisplayMsg "ERROR: Error retrieving file \"$ftp(RemoteFilename)\"!"
				       uplevel #0 "set closed 0"
				     }
			  }
			}
		get_close	{
			  switch $rc {
			    226     {
			  	       uplevel #0 "set closed 1"
			               set ftp(State) noop
			            }
			    default {
				       DisplayMsg "ERROR: Error retrieving file \"$ftp(RemoteFilename)\"!"
				       uplevel #0 "set closed 0"
				     }
			  }
			}

	}
}

#############################################################################
#
# Type --
#
# REPRESENTATION TYPE - Sets the file transfer type to ascii or binary.
# (exported)
#
# Arguments:
# type - 		specifies the representation type (ascii|binary)
# 

proc Type {type} {
variable ftp 

	set ftp(Type) $type

	set ftp(State) type
	StateHandler

	# wait for synchronization
	# synchronize with procedure StateHandler
	vwait closed 
}

#############################################################################
#
# NList --
#
# NAME LIST - This command causes a directory listing to be sent from
# server to user site.
# (exported)
# 
# Arguments:
# dir - 		The $dir should specify a directory or other system 
#			specific file group descriptor; a null argument 
#			implies the current directory. 
#
# Arguments:
# dir - 		directory to list 
# 
# Returns:
# sorted list of files or "-1" if listing fails

proc NList {{dir ""}} {
variable ftp 

	Type ascii

	set ftp(List) {}
	if { $dir == "" } {
		set ftp(Dir) ""
	} else {
		set ftp(Dir) " $dir"
	}

	set ftp(State) nlist
	StateHandler

	# wait for synchronization
	# synchronize with procedure HandleList and procedure StateHandler
	upvar #0 closed closed
	vwait closed
	
	if $closed {
		vwait closed
		unset ftp(Dir)
		return [lsort $ftp(List)]
	} else {
		Abort
		return -1
	}
}

#############################################################################
#
# List --
#
# LIST - This command causes a list to be sent from the server
# to user site.
# (exported)
# 
# Arguments:
# dir - 		If the $dir specifies a directory or other group of 
#			files, the server should transfer a list of files in 
#			the specified directory. If the $dir specifies a file
#			then the server should send current information on the
# 			file.  A null argument implies the user's current 
#			working or default directory.  
# 
# Returns:
# list of files or "-1" if listing fails

proc List {{dir ""}} {
variable ftp 

	Type ascii

	set ftp(List) {}
	if { $dir == "" } {
		set ftp(Dir) ""
	} else {
		set ftp(Dir) " $dir"
	}

	set ftp(State) list
	StateHandler

	# wait for synchronization
	# synchronize with procedure HandleList and procedure StateHandler
	upvar #0 closed closed
	vwait closed
	
	if $closed {
		vwait closed
		unset ftp(Dir)
		return [split $ftp(List) "\n"]
	} else {
		Abort
		return -1
	}
}

#############################################################################
#
# Pwd --
#
# PRINT WORKING DIRECTORY - Causes the name of the current working directory.
# (exported)
# 
# Arguments:
# None.
# 
# Returns:
# current directory name

proc Pwd {} {
variable ftp 

	set ftp(Dir) {}

	set ftp(State) pwd
	StateHandler

	# wait for synchronization
	# synchronize with procedure StateHandler
	vwait closed 
	
	return $ftp(Dir)
}

proc Site { cmd } {
variable ftp

        set ftp(Site) ""

        set ftp(State) site
	set ftp(Cmd) $cmd
        StateHandler

        upvar #0 closed closed
        vwait closed
	return $ftp(Site)
}

proc Quote { cmd } {
variable ftp

        set ftp(Quote) ""

        set ftp(State) quote
	set ftp(Cmd) $cmd
        StateHandler

        upvar #0 closed closed
        vwait closed
	return $ftp(Quote)
}

proc Quotew { cmd } {
variable ftp

        set ftp(Quote) ""

        set ftp(State) quotew
	set ftp(Cmd) $cmd
        StateHandler

        upvar #0 closed closed
        vwait closed
	return $ftp(Quote)
}

#############################################################################
#
# Cd --
#   
# CHANGE DIRECTORY - Sets the working directory on the server host.
# (exported)
# 
# Arguments:
# dir -			pathname specifying a directory

proc Cd {dir} {
variable ftp 

	set ftp(Dir) $dir

	set ftp(State) cd
	StateHandler

	# wait for synchronization
	# synchronize with procedure StateHandler
	vwait closed 

	unset ftp(Dir)
}

#############################################################################
#
# MkDir --
#
# MAKE DIRECTORY - This command causes the directory specified in the $dir
# to be created as a directory (if the $dir is absolute) or as a subdirectory
# of the current working directory (if the $dir is relative).
# (exported)
# 
# Arguments:
# dir -			new directory name

# The ftp mdir command
proc MkDir {dir} {
variable ftp 

	set ftp(Dir) $dir

	set ftp(State) mkdir
	StateHandler

	# wait for synchronization
	# synchronize with procedure StateHandler
	vwait closed 

	unset ftp(Dir)
}

#############################################################################
#
# RmDir --
#
# REMOVE DIRECTORY - This command causes the directory specified in $dir to 
# be removed as a directory (if the $dir is absolute) or as a 
# subdirectory of the current working directory (if the $dir is relative).
# (exported)
#
# Arguments:
# dir -			directory name

proc RmDir {dir} {
variable ftp 

	set ftp(Dir) $dir

	set ftp(State) rmdir
	StateHandler

	# wait for synchronization
	# synchronize with procedure StateHandler
	vwait closed 

	unset ftp(Dir)
}

#############################################################################
#
# Delete --
#
# DELETE - This command causes the file specified in $file to be deleted at 
# the server site.
# (exported)
# 
# Arguments:
# file -			file name

proc Delete {file} {
variable ftp 

	set ftp(File) $file

	set ftp(State) delete
	StateHandler

	# wait for synchronization
	# synchronize with procedure StateHandler
	vwait closed 

	unset ftp(File)
}

#############################################################################
#
# Rename --
#
# RENAME FROM TO - This command causes the file specified in $from to be 
# renamed at the server site.
# (exported)
# 
# Arguments:
# from -			specifies the old file name of the file which 
#				is to be renamed
# to -				specifies the new file name of the file 
#				specified in the $from agument

proc Rename {from to} {
variable ftp 

	set ftp(RenameFrom) $from
	set ftp(RenameTo) $to

	set ftp(State) rename
	StateHandler

	# wait for synchronization
	# synchronize with procedure StateHandler
	vwait closed 

	unset ftp(RenameFrom)
	unset ftp(RenameTo)
}

#############################################################################
#
# ElapsedTime --
#
# Gets the elapsed time for file transfer
# 
# Arguments:
# stop_time - 		ending time

proc ElapsedTime {stop_time} {
variable ftp

	set elapsed [expr $stop_time - $ftp(Start_Time)]
	if { $elapsed == 0 } { set elapsed 1}
	set persec [expr $ftp(Total) / $elapsed]
	DisplayMsg "$ftp(Total) bytes sent in $elapsed seconds ($persec Bytes/s)"
}

#############################################################################
#
# PUT --
#
# STORE DATA - Causes the server to accept the data transferred via the data 
# connection and to store the data as a file at the server site.  If the file
# exists at the server site, then its contents shall be replaced by the data
# being transferred.  A new file is created at the server site if the file
# does not already exist.
# (exported)
#
# Arguments:
# source -			local file name
# dest -			remote file name, if unspecified, ftp assigns
#				the local file name.
# Returns:
# 0 -			file not stored
# 1 - 			OK

proc Put {source {dest ""}} {
variable ftp 

	if ![file exist $source] {
		DisplayMsg "ERROR: File \"$source\" not exist"
		return 0
     	}
			
	if { $dest == "" } {
		set dest $source
	}

	set ftp(LocalFilename) $source
	set ftp(RemoteFilename) $dest

	set ftp(SourceCI) [open $ftp(LocalFilename) r]
	if { $ftp(Type) == "ascii" } {
		fconfigure $ftp(SourceCI) -buffering line -blocking 1
	} else {
		fconfigure $ftp(SourceCI) -buffering line -translation binary -blocking 1
	}

	set ftp(State) put
	StateHandler

	# wait for synchronization
	# synchronize with procedure HandleList and procedure StateHandler
	upvar #0 closed closed
	vwait closed
	
	if $closed {
		vwait closed
		ElapsedTime [clock seconds]
		return 1
	} else {
		Abort
		return 0
	}

}

#############################################################################
#
# Get --
#
# RETRIEVE DATA - Causes the server to transfer a copy of the specified file
# to the local site at the other end of the data connection.
# (exported)
#
# Arguments:
# source -			remote file name
# dest -			local file name, if unspecified, ftp assigns
#				the remote file name.
# Returns:
# 0 -			file not retrieved
# 1 - 			OK

proc Get {source {dest ""}} {
variable ftp 

	if { $dest == "" } {
		set dest $source
	}

	set ftp(RemoteFilename) $source
	set ftp(LocalFilename) $dest

	set rc [catch {set ftp(DestCI) [open $ftp(LocalFilename) w]} msg]
	if { $rc != 0 } {
		DisplayMsg "ERROR: $msg"
		return
	}

	if { $ftp(Type) == "ascii" } {
		fconfigure $ftp(DestCI) -buffering line -blocking 1
	} else {
		fconfigure $ftp(DestCI) -buffering line -translation binary -blocking 1
	}

	set ftp(State) get
	StateHandler

	# wait for synchronization
	# synchronize with procedure HandleList and procedure StateHandler
	upvar #0 closed closed
	vwait closed
	
	if $closed {
		vwait closed
		ElapsedTime [clock seconds]
		return 1
	} else {
		Abort
		return 0
	}

}

#############################################################################
#
# Abort -- 
#
# ABORT - Tells the server to abort the previous ftp service command and 
# any associated transfer of data. The control connection is not to be 
# closed by the server, but the data connection must be closed.
#
# Arguments:
# None.

proc Abort {} {
variable ftp

	if ![info exists ftp(State)] {
		return
	}

	set ftp(State) abort
	StateHandler

	# wait for synchronization
	# synchronize with procedure StateHandler
	vwait aborted 

	catch {close $ftp(DataSock)}
}

#############################################################################
#
# Close -- 
#
# Terminates a ftp session and if file transfer is not in progress, the server
# closes the control connection.  If file transfer is in progress, the 
# connection will remain open for result response and the server will then
# close it. 
# (exported)
# 
# Arguments:
# None.

proc Close {} {
variable ftp

	if ![info exists ftp(State)] {
		return
	}

	set ftp(State) quit
	StateHandler

	# wait for synchronization
	# synchronize with procedure StateHandler
	vwait closed 

	catch {close $ftp(CtrlSock)}
	unset ftp(State) 
}

#############################################################################
#
# Open --
#
# Starts the ftp session and sets up a ftp control connection.
# (exported)
# 
# Arguments:
# server - 		The ftp server hostname.
# user -		A string identifying the user. The user identification 
#			is that which is required by the server for access to 
#			its file system.  
# passwd -		A string specifying the user's password.
# 
# Returns:
# 0 -			Not logged in
# 1 - 			User logged in

proc Open {server port user passwd} {
variable ftp

	if [info exists ftp(State)] {
       		DisplayMsg "C: Mmh, another attempt to open a new connection? There is already a hot wire!"
		return 0
	}

	set ftp(User) 		$user
	set ftp(Passwd) 	$passwd
	set ftp(RemoteHost) 	$server
	set ftp(Port)           $port
	set ftp(LocalHost) 	[info hostname]
	set ftp(DataPort) 	0
	set ftp(Type) 		{}
	set ftp(Chunk) 		4096	
	set ftp(Timeout) 	1000	

	set ftp(State) 		user
	
	# No call of StateHandler is required at this time.
	# StateHandler at first time is called automatically
	# by a fileevent for the control channel.

	# Try to open a control connection
	if ![OpenControlConn] { return 0 }

	# waits for synchronization
	vwait login
		
	# global variable "login" contains login state
	# will be changed in procedure StateHandler
	#   0 ... Not logged in
	#   1 ... User logged in
	upvar #0 login login
	
	# default mode is ascii
	# close connection if not logged in
	if $login {
		Type ascii
	} else {
		Close
	}
	
	return $login
}

#############################################################################
#
# CopyNext --
#
# recursive background copy procedure for ascii/binary file I/O
# 
# Arguments:
# bytes - 		indicates how many bytes were written on $ftp(DestCI)

proc CopyNext {bytes} {
variable ftp 

	# summary bytes		
	incr ftp(Total) $bytes

	if [eof $ftp(SourceCI)] {

		# Close channels
		close $ftp(DestCI)
		close $ftp(SourceCI)
		DisplayMsg "D: Port closed"
		uplevel #0 "set closed 1"
		
	} else {
		fcopy $ftp(SourceCI) $ftp(DestCI) -command CopyNext -size $ftp(Chunk)

	}

}

#############################################################################
#
# HandleList --
#
# Handles ascii/binary data transfer for Put and Get 
# 
# Arguments:
# sock - 		socket name (data channel)

proc HandleData {sock} {
variable ftp 

	# Turn off any fileevent handlers
	fileevent $sock writable {}		
	fileevent $sock readable {}
	
	# perform fcopy	
	set ftp(Total) 0
	set ftp(Start_Time) [clock seconds]
	fcopy $ftp(SourceCI) $ftp(DestCI) -command CopyNext -size $ftp(Chunk)
}

#############################################################################
#
# HandleList --
#
# Handles ascii data transfer for list commands
# 
# Arguments:
# sock - 		socket name (data channel)

proc HandleList {sock} {
variable ftp 

	if ![eof $sock] {
		set buffer [read $sock]
		if { $buffer != "" } {
			set ftp(List) [append ftp(List) $buffer]
		}	
	} else {
   		close $sock
 		DisplayMsg "D: Port closed"
		uplevel #0 "set closed 1"
	} 
}

#############################################################################
#
# InitDataConn --
#
# Configures new data channel for connection to ftp server 
# ATTENTION! The new data channel "sock" is not the same as the 
# server channel "ftp(DataSock)".
# 
# Arguments:
# sock -		the name of the new channel
# addr -		the address, in network address notation, 
#			of the client's host,
# port -		the client's port number

proc InitDataConn {sock addr port} {
variable ftp

	# If the new channel is accepted, the server channel will closed
	close $ftp(DataSock)

	# Configure translation mode
	if { $ftp(Type) == "ascii" } {
		fconfigure $sock -buffering line -blocking 1
	} else {
		fconfigure $sock -buffering line -translation binary -blocking 1
	}

	# assign fileevent handlers, source and destination CI (Channel Identifier)
	switch -regexp $ftp(State) {

		^list {
			  fileevent $sock readable [list [namespace current]::HandleList $sock]
			  set ftp(SourceCI) $sock		  
			}
		
		^get	{
			  fileevent $sock readable [list [namespace current]::HandleData $sock]
			  set ftp(SourceCI) $sock			  
			}

		^put {
			  fileevent $sock writable [list [namespace current]::HandleData $sock]
			  set ftp(DestCI) $sock			  
			}
	}

	DisplayMsg "D: Connection from $addr:$port"
}

#############################################################################
#
# OpenDataConn --
#
# Opens a ftp data connection
# 
# Arguments:
# None.
# 
# Returns:
# 0 -			no connection
# 1 - 			connection established

proc OpenDataConn {} {
variable ftp

	# Port address 0 is used to give the server the responsibility 
	# of getting free new port addresses for every data transfer.
	set rc [catch {set ftp(DataSock) [socket -server InitDataConn 0]} msg]
	if { $rc != 0 } {
       		DisplayMsg "D: $msg"
       		return 0
	}

	# get a new local port address for data transfer and convert it to a format
	# which is useable by the PORT command
	set p [lindex [fconfigure $ftp(DataSock) -sockname] 2]
	DisplayMsg "D: Port is $p"
	set ftp(DataPort) "[expr "$p / 256"],[expr "$p % 256"]"

	return 1
}

#############################################################################
#
# OpenControlConn --
#
# Opens a ftp control connection
# 
# Arguments:
# None.
# 
# Returns:
# 0 -			no connection
# 1 - 			connection established

proc OpenControlConn {} {
variable ftp

	# open a control channel
        set rc [catch {set ftp(CtrlSock) [socket $ftp(RemoteHost) $ftp(Port)]} msg]
	if { $rc != 0 } {
       		DisplayMsg "C: No connection to server!"
		if { [info exist ftp(Debug)] && $ftp(Debug)} {
			DisplayMsg "DEBUG: [list $msg]"
		}
       		unset ftp(State)
       		return 0
	}
	
	# configure control channel
	fconfigure $ftp(CtrlSock) -buffering line -blocking 0
        fileevent  $ftp(CtrlSock) readable [list [namespace current]::StateHandler $ftp(CtrlSock)]
	
	# prepare local ip address for PORT command (convert pointed format to comma format)
	set ftp(LocalAddr) [lindex [fconfigure $ftp(CtrlSock) -sockname] 0]
	regsub -all "\[.\]" $ftp(LocalAddr) "," ftp(LocalAddr) 

	# report ready message
	set peer [fconfigure $ftp(CtrlSock) -peername]
	DisplayMsg "C: Connection from [lindex $peer 0]:[lindex $peer 2]"
	
	return 1
}


# not forgotten close-brace (end of namespace)
}






