###############################################################
#                                      __  __  _              #
#            ___  ____ __  _ ___  ___ / /_ \ \/ /             #
#           / _ \' __/ _ `/ / -_)/ __/  _/  )  (              #
#          / .__/_/  \___/ /\___/\__/\__/  /_/\_\             #
#         /_/ -------- /__/ ---------------------             #
#                                                             #
#      Explainer v1.00 by |storm| <storm@princess.dk>         #
#             http://www.projectx.princess.dk/                #
###############################################################

### -----------------------------------------------------------
###                       Description
### -----------------------------------------------------------
###
### This tcl script allows you to add and delete terms and 
### explanations/descriptions to a database. At any time you 
### can query the database for an explanation, which will be 
### displayed in the channel. (See 'Command reference' for a 
### quick overview of all available commands.) This tcl script
### can be used in help or support channels where you can set
### up a database of standard replies on frequently asked 
### questions.  In non-support channels it is often used for
### entertainment by adding funny explanations to the database.
###
### Before we decided to release this script, we thoroughly
### tested it on several different version of Eggdrop, going 
### from version 1.1.5 to 1.3.8.  Note that the script will
### *not* work properly on a Win95/Win98/NT port of Eggdrop.
###
### Bugreports and constructive critism for features to add
### can be send to storm@princess.dk.  If you provide us with
### ideas for extra features, you have a big chance we'll add
### them for you in a next version/release of this script.
### Also, if you made a valuable explanation file, send it to
### us and we will distribute it (incl. some credits) with this
### tcl script.

### -----------------------------------------------------------
###                   User-defined settings
### -----------------------------------------------------------

### Set the command character that will proceed all public 
### commands.
###   Example: 
###   If you set it to "!", the command to list all available
###   explanations would be "!ls".  On the other hand, if you
###   set the command character (cc) to "expl.", the same 
###   command would be "expl.ls".
set cc "!"

### Set the file (and the path, relative to the eggdrop's root
### directory) that should be used to retrieve and store all
### data:
set explfile "./botnick.expl"

### -----------------------------------------------------------
###                     Command reference
### -----------------------------------------------------------
###
###  !expl <term>          -   explain    -   no flag required
###  !add <term> <expl>    -   add        -   +f flag required
###  !del <term>           -   delete     -   +f flag required
###  !ls                   -   list       -   no flag required
###
###  Note 1: Only users with the +f flag (friend) set, are able
###          (allowed) to add and delete explanations.
###  Note 2: In the above command reference, the command 
###          character is set to "!", of course this may differ
###          depending on your personal settings above.

### -----------------------------------------------------------
###                       About ProjectX
### -----------------------------------------------------------
###
### We are an organised group of scripters determined to help
### each other create, develop and refine scripts. We bundle our 
### knowledge, so that the inexperienced scripters may benefit 
### from our experienced crew-members and our experienced 
### members benefit from the enthousiasm and fresh insights 
### from our less-experienced members. We are not trying to 
### create the 'ULTIMATE' script, there is no such thing. We
### concentrate on creating good, reliable, compatible and
### useful scripts for eggdrops.
### You can contact ProjectX in the following ways:
###  -  Visit our homepage..: http://www.projectx.princess.dk/
###  -  Vist us on IRC......: #projectx (Galaxynet)
###  -  Drop us an e-mail...: px-info@princess.dk


###############################################################
########### Don't change anything below this line! ############
###############################################################

set ver "1.00"

bind pub - ${cc}expl pub:expl
bind pub f ${cc}add pub:expl.add
bind pub f ${cc}del pub:expl.del
bind pub - ${cc}ls pub:expl.ls

proc pub:expl {nick uhost handle chan arg} {
  global explfile cc
  putcmdlog "#${handle}# expl"
  set expl [expl:get [lindex $arg 0]]
  if {$expl!="0"} { 
    puthelp "PRIVMSG $chan :[lindex $arg 0]: $expl"
  } else { 
    puthelp "NOTICE $nick :\[expl] Error: term not recognised.  Type '${cc}ls' for a list of all available explanations." 
  }
}

proc pub:expl.add {nick uhost handle chan arg} {
  global explfile cc
  putcmdlog "#${handle}# add expl"
  if {([lindex $arg 0]!="") && ([lindex $arg 1]!="")} {
    if {[string length [lindex $arg 0]]<25} {
      if {![string match "*;*" $arg]} {
        if {[expl:add [lindex $arg 0] [lrange $arg 1 end] ${handle}!${uhost}]} {
          puthelp "NOTICE $nick :\[add\] Explanation has been sucessfully added."
        } else {
          puthelp "NOTICE $nick :\[add\] Error: unable to add a duplicate term.  Type '${cc}del [lindex $arg 0]' if you want to delete the existing explanation first."
        }
      } else {
        puthelp "NOTICE $nick :\[add\] Error: the use of the ';'-character is restricted."
      }
    } else {
      puthelp "NOTICE $nick :\[add\] Error: term too long."
    }
  } else {
    puthelp "NOTICE $nick :\[add\] Usage: ${cc}add <term> <expl>"
  }
}

proc pub:expl.del {nick uhost handle chan arg} {
  global explfile cc
  putcmdlog "#${handle}# del expl"
  if {[lindex $arg 0]!=""} {
    if {[expl:del [lindex $arg 0]]} {
      puthelp "NOTICE $nick :\[del\] Explanation has been deleted."
    } else {
      puthelp "NOTICE $nick :\[del\] Error: term not recognised.  Type '${cc}ls' for a list of all available explanations."
    }
  } else {
    puthelp "NOTICE $nick :\[del\] Usage: ${cc}del <term>"
  }
}

proc pub:expl.ls {nick uhost handle chan args} {
  global explfile cc
  putcmdlog "#${handle}# ls expl"
  set list ""
  set fd [open $explfile r]
  puthelp "NOTICE $nick :\[ls\] \037List of all available terms:\037"
  while {![eof $fd]} {
    set term [expl:term [gets $fd]]
    if {($term!="-1") && ($term!="")} {
      while {[string length $term]<10} { append term " " }
      append list "  $term"
    }
    if {[string length $list]>45} { 
      puthelp "NOTICE $nick :\[ls\]  $list"
      set list ""
    }
  }
  close $fd
  if {$list!=""} { puthelp "NOTICE $nick :\[ls\]  $list" }
  if {[matchattr $handle f]} {
    puthelp "NOTICE $nick :\[ls\] See also: '${cc}expl <term>', '${cc}add <term> <expl>' and '${cc}del <term>'."
  } else {
    puthelp "NOTICE $nick :\[ls\] See also: '${cc}expl <term>'."
  }
} 

proc expl:get {term} {
  global explfile
  set fd [open $explfile r]
  while {![eof $fd]} {
    set input [gets $fd]
    if {[expl:term $input]==[string tolower $term]} { 
      close $fd
      return [string range $input [expr [string first ";" $input]+1] [expr [string last ";" $input]-1]]
    }
  }
  close $fd
  return 0
}

proc expl:add {term expl user} {
  global explfile
  set added 0
  set ifd [open $explfile r]
  set ofd [open ${explfile}.tmp w]
  while {![eof $ifd]} {
    set fln [gets $ifd]
    set i [string compare [expl:term $fln] [string tolower $term]]
    if {($i==1) && (!$added)} {
      puts $ofd "[string tolower $term];$expl;$user"
      set added 1
    }
    if {$i==0} { 
       close $ifd
       close $ofd
       exec rm ${explfile}.tmp
       return "-1"
    }
    puts $ofd $fln
  }
  if {!$added} { puts $ofd "[string tolower $term];$expl;$user" }
  close $ifd
  close $ofd
  exec mv ${explfile}.tmp $explfile
  return 1
}

proc expl:del {term} {
  global explfile
  set rval 0
  set ifd [open $explfile r]
  set ofd [open ${explfile}.tmp w]
  while {![eof $ifd]} {
    set fln [gets $ifd]
    if {[expl:term $fln]!=[string tolower $term]} { 
      puts $ofd $fln
      set rval 1
    }
  }
  close $ifd
  close $ofd
  exec mv ${explfile}.tmp $explfile
  return $rval
}

proc expl:term {fstring} {
  return [string range $fstring 0 [expr [string first ";" $fstring]-1]]
}

if {![file exists $explfile]} {
  set fd [open $explfile w]
  close $fd
}

putlog "\[ProjectX\] Explainer v$ver by |storm| <storm@princess.dk> loaded."
