# quoteme 1.1 by k-rist (k-rist@rsn.hk-r.se)
#
# This script basically stores quotes and sends a random quote to the channel.
# Inspired by the quotes 1.0 by blue (blue@tical.net).
#
# Intended users are eggdrop 1.3.2x bots.
#
# Commands:
# quoteme           - enables or disables this feature, only available
#                   - for owners (n) via dcc
# quote [number]    - sends a random quote to the channel, if [number] is
#                   - specified, that quote in the quotemefile will be quoted
#                   - first quote is number 1
# addquote <quote>  - adds <quote> to the quotemefile
# delquote <number> - attempts to delete the quote with <number>
# lsquotes [number] - lists quotes. If [number] is specified, start listing at
#                   - that quote, otherwise, start at first quote.
#
# The respones from the commands "addquote", "delquote" and "lsquotes" will
# be sent to the user by a notice, if used directly in the channel.
#
# User flags required:
# Using quote in public requires no flag set (-)
# Using addquote, lsquotes in public requires the user to be 
#  friend with the channel or to be a global friend (f)
# Using quote, addquote, lsquotes in msg or dcc requires the user to be friend
#  with the channel or to be a global friend (f)
# Using delquote at public, msg or dcc requires the owner flag set (n)
# Using quoteme at dcc requires the owner flag set (only available via dcc)
#
# Using a command in public or msg mode, the command must be preceded by a !
# For example: !quote
# Using a command in dcc mode, the command must be preceded by a .
# For example: .quote
#
# Wishlist:
# Better errormessages
# Instead of specify how many, specify which quotes that are listed
# Fix the "Tcl error xxx_quote: integer value too large to represent" error 
#
###############################################################################

# Which prefix should the quote files have?
set quotemeprefix ".quoteme"

# Which channels to send quotes to?
set quotemechannels "#ud"

# Allow the !quote command via public? (0/1)
set quotepub 1
# Allow !quote commands via msg? (0/1)
set quotemsg 1
# Allow .quote commands via dcc? (0/1)
set quotedcc 1

# Allow the !addquote command via public? (0/1)
set addquotepub 1
# Allow !addquote commands via msg? (0/1)
set addquotemsg 1
# Allow .addquote commands via dcc? (0/1)
set addquotedcc 1

# Allow the !delquote command via public? (0/1)
set delquotepub 0
# Allow !delquote commands via msg? (0/1)
set delquotemsg 0
# Allow .delquote commands via dcc? (0/1)
set delquotedcc 1

# Allow the !lsquotes command via public? (0/1)
set lsquotespub 1
# Allow !lsquotes commands via msg? (0/1)
set lsquotesmsg 1
# Allow .lsquotes commands via dcc? (0/1)
set lsquotesdcc 1

# Amount of quotes to list when using !lsquotes at public (0/x)
# When a listing occurs in public or msg, all other messages
# sent to public and msg is on hold.
# Setting this to 0 turns it off
set publsamount 5

# Amount of quotes to list when using !lsquotes at msg (0/x)
# When a listing occurs in public or msg, all other messages
# sent to public and msg is on hold.
# Setting this to 0 turns it off
set msglsamount 5

# Amount of quotes to list when using .lsquotes at dcc (0/x)
# This listing does not put other messages on hold
# Setting this to 0 turns it off
set dcclsamount 0

# Enable this feature (script)? (0/1)
# 0 disables the feature, 1 enables the feature
set quoteme 1





##################################################
# You should NOT change anything below this line #
##################################################

set quotemeversion "quoteme 1.1"

if {$quotepub} { bind pub - !quote pub_quote }
if {$addquotepub} { bind pub - !addquote pub_addquote }
if {$delquotepub} { bind pub n !delquote pub_delquote }
if {$lsquotespub} { bind pub - !lsquotes pub_lsquotes }

if {$quotemsg} { bind msg - !quote msg_quote }
if {$addquotemsg} { bind msg - !addquote msg_addquote }
if {$delquotemsg} { bind msg n !delquote msg_delquote }
if {$lsquotesmsg} { bind msg - !lsquotes msg_lsquotes }

if {$quotedcc} { bind dcc - quote dcc_quote }
if {$addquotedcc} { bind dcc - addquote dcc_addquote }
if {$delquotedcc} { bind dcc n delquote dcc_delquote }
if {$lsquotesdcc} { bind dcc - lsquotes dcc_lsquotes }

bind dcc o quoteme dcc_quoteme

proc pub_quote {nick uhost handle channel arg} {
    global quoteme
    if {$quoteme} {
	quote "pub" $nick $handle [concat $channel $arg]
    }
    return 1
}

proc pub_addquote {nick uhost handle channel arg}  {
    global quoteme
    if {$quoteme} {
	addquote "pub" $nick $handle [concat $channel $arg]
    }
    return 1
}

proc pub_delquote {nick uhost handle channel arg} {
    global quoteme
    if {$quoteme} {
	delquote "pub" $nick $handle [concat $channel $arg]
    }
    return 1
}

proc pub_lsquotes {nick uhost handle channel arg} {
    global quoteme
    if {$quoteme} {
	lsquotes "pub" $nick $handle [concat $channel $arg]
    }
    return 1
}

proc msg_quote {nick uhost handle arg} {
    global quoteme
    if {$quoteme} {
	quote "msg" $nick $handle $arg
    }
    return 1
}

proc msg_addquote {nick uhost handle arg} {
    global quoteme
    if {$quoteme} {
	addquote "msg" $nick $handle $arg
    }
    return 1
}

proc msg_delquote {nick uhost handle arg} {
    global quoteme
    if {$quoteme} {
	delquote "msg" $nick $handle $arg
    }
    return 1
}

proc msg_lsquotes {nick uhost handle arg} {
    global quoteme
    if {$quoteme} {
	lsquotes "msg" $nick $handle $arg
    }
    return 1
}

proc dcc_quote {handle idx arg} {
    global quoteme
    if {$quoteme} {
	quote "dcc" $idx $handle $arg
    }
    return 1
}

proc dcc_addquote {handle idx arg}  {
    global quoteme
    if {$quoteme} {
	addquote "dcc" $idx $handle $arg
    }
    return 1
}

proc dcc_delquote {handle idx arg} {
    global quoteme
    if {$quoteme} {
	delquote "dcc" $idx $handle $arg
    }
    return 1
}

proc dcc_lsquotes {handle idx arg} {
    global quoteme
    if {$quoteme} {
	lsquotes "dcc" $idx $handle $arg
    }
    return 1
}

proc dcc_quoteme {handle idx arg} {
    global quoteme
    global quotemeversion
    if {$quoteme} {
        set quoteme 0
        set answer "$quotemeversion disabled - quotes can not be sent"
    } else {
        set quoteme 1
        set answer "$quotemeversion enabled - quotes can be sent"
    }
    putdcc $idx $answer
    return 1
}

# Sends (if possible) a quote to the channel
# return 1 if ok, 0 otherwise
proc quote {type who handle arg} {
    global quotemeprefix
    set channel [string tolower [getchannel $arg]]
    if {[checkchannel $channel]} {
	if {$type != "pub"} {
	    if {![checkuser $handle $channel]} {
		return 0
	    }
	}
	set arg [lreplace $arg 0 0]
	set channelfile $quotemeprefix$channel
	if {[checkfile $channelfile "r"]} {
	    set quotes [getquotes $channelfile]
	    if {[expr [llength $quotes] > 0]} {
		# Either get a random quote, or a specified quote 
		if {$arg != ""} {
		    set arg [checkarg $arg]
		    if {$arg == ""} {
			if {$type == "pub"} {
			    sendanswer $type $who "Usage: !quote \[number\]"
			} else {
			    sendanswer $type $who "Usage: !quote <#channel> \[number\]"
			}
			return 1
		    }
		set row [expr $arg - 1]
		} else {
		    set row [rand [llength $quotes]]
		}
		
		# Say the quote
		if {[expr $row >= 0] && [expr $row < [llength $quotes]]} {
		    set quote [lindex $quotes $row]
		    if {$quote != ""} {
			set row [expr $row + 1]
			putserv "PRIVMSG $channel :\002($row) $quote\002" 
			return 1
		    }
		}
	    }
	}
    } else {
	sendanswer $type $who "Bad channel name or it is not being monitored ($channel)"
	if {$type == "pub"} {
	    sendanswer $type $who "Usage: !quote \[number\]"
	} else {
	    sendanswer $type $who "Usage: !quote <#channel> \[number\]"
	}
	return 0
    }
    sendanswer $type $who "quote failed"
    return 0
}

proc addquote {type who handle arg} {
    global quotemeprefix
    set channel [string tolower [getchannel $arg]]
    if {[checkchannel $channel]} {
	if {![checkuser $handle $channel]} {
	    return 0
	}
	set arg [lreplace $arg 0 0]
	set channelfile $quotemeprefix$channel
	if {[file exists $channelfile]} {
	    if {![file writable $channelfile] || [file isdir $channelfile]} {
		sendanswer $type $who "addquote failed"
		return 0
	    }
	    set quotes [getquotes $channelfile]
	} else {
	    set quotes ""
	}
	
	if {$arg != ""} {
	    if {![expr [lsearch -exact $quotes $arg] >= 0]} {
		set fileio [open $channelfile "a"]
		puts $fileio $arg
		flush $fileio
		close $fileio
		sendanswer $type $who "Quote added."
		return 1
	    } else {
		sendanswer $type $who "Quote already exists."
		return 0
	    }
	}
    } else {
	sendanswer $type $who "Bad channel name or it is not being monitored ($channel)"
    }    
    if {$type == "pub"} {
	sendanswer $type $who "Usage: !addquote <quote>"
    } else {
	sendanswer $type $who "Usage: !addquote <#channel> <quote>"
    }
    return 0
}

proc delquote {type who handle arg} {
    global quotemeprefix
    set channel [string tolower [getchannel $arg]]
    if {[checkchannel $channel]} {
	set arg [lreplace $arg 0 0]
	set channelfile $quotemeprefix$channel
	if {[checkfile $channelfile "w"]} {
	    if {$arg != ""} {
		set arg [checkarg $arg]
	    }
	    if {$arg == ""} {
		if {$type == "pub"} {
		    sendanswer $type $who "Usage: !delqoute <number>"
		} else {
		    sendanswer $type $who "Usage: !delquote <#channel> <number>"
		}
		return 0
	    }
	    
	    set row [expr $arg - 1]
	    set oldquotes [getquotes $channelfile]
	    if {[expr $row >= 0] && [expr $row < [llength $oldquotes]]} {
		set newquotes [lreplace $oldquotes $row $row]
		set fileio [open $channelfile "w"]
		foreach quote $newquotes {
		    puts $fileio $quote
		}
		flush $fileio
		close $fileio
		sendanswer $type $who "Quote deleted"
		return 1
	    }
	}
    } else {
	sendanswer $type $who "Bad channel name or it is not being monitored ($channel)"
	if {$type == "pub"} {
	    sendanswer $type $who "Usage: !delqoute <number>"
	} else {
	    sendanswer $type $who "Usage: !delquote <#channel> <number>"
	}
	return 0
    }
    sendanswer $type $who "delquote failed"
    return 0
}

proc lsquotes {type who handle arg} {
    global quotemeprefix
    global publsamount
    global msglsamount
    global dcclsamount
    set channel [string tolower [getchannel $arg]]
    if {[checkchannel $channel]} {
	if {![checkuser $handle $channel]} {
	    return 0
	}
	set arg [lreplace $arg 0 0]
	set channelfile $quotemeprefix$channel
	if {[checkfile $channelfile "r"]} {
	    set quotes [getquotes $channelfile]
	    set newquotes ""
	    foreach quote $quotes {
		set index [lsearch -exact $quotes $quote]
		set row [expr $index + 1]
		set newquote [concat $row $quote]
		set newquotes [linsert $newquotes end $newquote]
	    }
	    
	    if {$arg != ""} {
		set arg [checkarg $arg]
		if {$arg == ""} {
		    if {$type == "pub"} {
			sendanswer $type $who "Usage: !lsquotes \[number\]"
		    } else {
			sendanswer $type $who "Usage: !lsquotes <#channel> \[number\]"
		    }
		    return 0
		}
		if {[expr $arg > [llength $newquotes]]} {
		    set newquotes ""
		} else {
		    # Since quotes start at 1, and we also want to list the specified
		    # quote, we hade to start removing from -2 and down.
		    set arg [expr $arg - 2]
		    while {[expr $arg >= 0]} {
			set newquotes [lreplace $newquotes $arg $arg]
			set arg [expr $arg - 1]
		    }
		}
	    }
	    
	    if {$type == "pub"} {
		set lsamount $publsamount
	    } elseif {$type == "msg"} {
		set lsamount $msglsamount
	    } elseif {$type == "dcc"} {
		set lsamount $dcclsamount
	    } else {
		return 0
	    }
	    set quotesamount [llength $newquotes]
	    if {[expr $quotesamount > $lsamount] && [expr $lsamount > 0]} {
		set quotesamount $lsamount
	    }

	    # Only list $lsamount amount of quotes
	    set index $lsamount
	    if {[expr $index <= 0] || [expr $index > $quotesamount]} {
		set index $quotesamount
	    }
            sendanswer $type $who "Quote list for $channel ([getamount $channelfile] total, listing $quotesamount):"
	    foreach quote $newquotes {
		if {[expr $index > 0]} {
		    sendanswer $type $who "$quote"
		    set index [expr $index - 1]
		} else {
		    break
		}
	    }
	    sendanswer $type $who "End of quotes list"
	    return 1
	}
    } else {
	sendanswer $type $who "Bad channel name or it is not being monitored ($channel)"
	if {$type == "pub"} {
	    sendanswer $type $who "Usage: !lsquotes \[number\]"
	} else {
	    sendanswer $type $who "Usage: !lsquotes <#channel> \[number\]"
	}
	return 0
    }
    sendanswer $type $who "lsquotes failed"
    return 0
}

# Checks if an arg is numeric (truncates the arg to first character)
# arg  - the arg to check
# return the new arg if arg is ok, "" if it failed
proc checkarg {arg} {
    set arg [lindex $arg 0]
    if {[regexp "^\[0-9\]+$" $arg]} {
	return $arg
    }
    return ""
}

# Checks if the channel is being monitored
# channel - the channel to check
# return 1 if ok, 0 otherwise
proc checkchannel {channel} {
    global quotemechannels
    if {$channel != ""} {
	foreach monitorchannel $quotemechannels {
	    set monitorchannel [string tolower $monitorchannel]
	    if {$channel == $monitorchannel} {
		return 1
	    }
        }
    }
    return 0
}

# Checks if the file exists and is a file (and not a directory)
# file - the file to check
# rw   - set "r" if file should be checked for readability
#      - set "w" if file should be checked for writability
# return 1 if file is ok, 0 if it failed
proc checkfile {file rw} {
    if {$file != ""} {
	if {[file exists $file] && [file isfile $file]} {
	    if {$rw == "r"} {
		if {[file readable $file]} {
		    return 1
		} else {
		    return 0
		}
	    } elseif {$rw == "w"} {
		if {[file writable $file]} {
		    return 1
		} else {
		    return 0
		}
	    } else {
		return 1
	    }
	}
    }
    return 0
}

# returns 1 if the user is friend with this channel (+f)
# or if the user is a global friend, 0 otherwise
# handle - handle of the user
# channel - the channel to check
proc checkuser {handle channel} {
    if {[matchattr $handle "+f|f" $channel]} {
	return 1
    }
    return 0
}

# Returns the amount of quotes if possible, 0 otherwise
# file - the quote file to fetch the quotes from
proc getamount {file} {
    if {[checkfile $file "r"]} {
	set quotes [getquotes $file]
	return [llength $quotes]
    }
    return 0
}

# Returns the file to use for the specific channel
# arg - the arg to filter
proc getchannel {arg} {
    global quotemechannels
    global quotemeversion
    set channel [lindex $arg 0]
    return $channel
}

# Returns a list with the quotes
proc getquotes {file} {
    set quotes ""
    set fileio [open $file "r"]
    while {![eof $fileio]} {
	set quote [gets $fileio]
	if {$quote != ""} {
            set quotes [linsert $quotes end $quote]
	}
    }
    close $fileio
    return $quotes
}

# Sends a message to the specified user
# return 1 if ok, 0 otherwise
proc sendanswer {type who message} {
    if {$type == "pub"} {
	set message [concat ":" $message]
	puthelp "NOTICE $who $message"
    } elseif {$type == "msg"} {
	set message [concat ":" $message]
	puthelp "PRIVMSG $who $message"
    } elseif {$type == "dcc"} {
	putdcc $who $message
    } else {
	return 0
    }
    return 1
}

putlog "$quotemeversion loaded."
