##########################
# Eggdrop Extensions by Vulture <vulture@abac.com>
###########
# Eggdrop Extensions is, as its name implies, a set of extensions to eggdrop.
# This includes many, many useful channel commands, private message commands,
# and DCC commands alike.
#
# WARNING: This script to be used with Eggdrop 1.3.x ONLY.
#
##########################
### INITIALIZATION ###
source eggdrop-ext.conf

set eggext_version 1.5

if { [info vars ctcp-version] == "ctcp-version" && ${ee-disguise-version} != "yes" } {
  set ctcp-version "${ctcp-version}+eggdrop extensions $eggext_version"
}

# Check if there's a botnet-nick set; if not, use bot's primary nick. The
# resulting variable, bnnick, can be used as a name identifier for the bot
# without defining one in this script.
if { [info vars botnet-nick] == "" } {
  set bnnick $nick
}
if { [info vars botnet-nick] != "" } {
  set bnnick ${botnet-nick}
}

# If the toolkit tcl isn't loaded, load 'er up. If it doesn't work, bitch in
# the log.
if {![info exists toolkit_loaded]} {
  catch {source scripts/toolkit.tcl}
  if {![info exists toolkit_loaded]} {
    catch {source toolkit.tcl}
    if {![info exists toolkit_loaded]} {
      putlog "Eggdrop Extensions WARNING: Can't load Tcl toolkit!"
    }
  }
}

# Reassign the +chan/-chan commands to a +m user, instead of +n.
unbind dcc n +chan *dcc:+chan
bind dcc m +chan *dcc:+chan
unbind dcc n -chan *dcc:-chan
bind dcc m -chan *dcc:-chan

# Reassign the whois command if we're supposed to.
if { $all_whois == "yes" } {
  unbind dcc o whois *dcc:whois
  bind dcc - whois *dcc:whois
}

# Reassign msg/say/act dcc commands to masters...that way global ops can't
# send bogus shit to other bots or users.
unbind dcc o say *dcc:say
bind dcc m say *dcc:say
unbind dcc o act *dcc:act
bind dcc m act *dcc:act
unbind dcc o msg *dcc:msg
bind dcc m msg *dcc:msg

### USEFUL PROCEDURES ###
# a fixed validchan... drop-in replaceable for those versions of eggdrop that
# happen to have a buggy `validchan' command, but useful because you can use
# vldchan on both eggdrops with this problem and those without
proc vldchan {channel} {
  if { [lsearch [string tolower [channels]] [string tolower $channel]] != -1 } {
    return 1
  }
  return 0
}

# sends a message to your subnet
proc putsubnet {msg} {
  global subnet-bots bot_subnet
  if { $bot_subnet == "" } {
    foreach tgt ${subnet-bots} {
      putbot $tgt $msg
    }
  }
  if { $bot_subnet != "" } {
    putallbots $msg
  }
  return 0
}

# validates a subnet message
proc subnetvalid {bot cmd arg} {
  global subnet-bots bot_subnet
  if { [lindex $arg 0] != "NEW" && $bot_subnet != "" } {
    if { [lindex $arg 0] != $bot_subnet } {
      return -1
    }
  }
  if { [lindex $arg 0] != "NEW" && $bot_subnet == "" } {
    return -1
  }
  if { [lsearch ${subnet-bots} $bot] < 1 } {
    return -1
  }
  return 1
}

# These two are for compatibility with older scripts that expect user-get and
# user-set to be there.
proc user-get {handle key} {
  return [getuser $handle XTRA $key]
}

proc user-set {handle key data} {
  setuser $handle XTRA $key $data
}

# Restores your saved console.
proc restore_console {handle idx} {
  global force-channel info-party internal-store
  if {${internal-store} != "yes"} {
    set cons [user-get $handle console]
    set conchan [user-get $handle conchan]
  }
  if {${internal-store} == "yes"} {
    set cons [getuser $handle console]
    set conchan [lindex $cons 0]
  }
  if {![matchattr $handle m]} {
    foreach x {c x r o} {
      regsub -all $x $cons "" cons
    }
  }
  if {$cons != ""} {
    if {$conchan != ""} {
      console $idx [lindex $conchan 0] [lindex $cons 0]
    }
  }
  set echo [user-get $handle echo]
  if {$echo != ""} { echo $idx $echo }
  set chan [user-get $handle chan]
  if {$chan == ""} {
     set chan ${force-channel}
  }
  setchan $idx $chan

  set info [getuser $handle INFO]
  if {${info-party} && $info != ""} {
     #dccputchan $chan "\[$handle\] $info"
  }
}

# Saves your console.
proc save_console {handle idx} {
  global internal-store
  set cons [console $idx]
  if {${internal-store} != "yes"} {
    user-set $handle conchan [lindex $cons 0]
    user-set $handle console [lindex $cons 1]
    user-set $handle echo [echo $idx]
    user-set $handle chan [getchan $idx]
  }
  if {${internal-store} == "yes"} {
    setuser $handle console "$cons [strip $idx] [echo $idx] [page $idx] 0"
  }
}

# Check if a handle is logged in under a nick!host.
proc isloggedin {hand host} {
  # if they're not a valid user, return no
  if { ![validuser $hand] } {
    return 0
  }
  # if their login host mismatches their current nick!user@host (which may be
  # blank, indicating that they're not logged in), no
  if { [getuser $hand xtra login-host] != $host && [getuser $hand xtra login-host] != [string range $host [expr [string first ! $host] + 1] end] } {
    return 0
  }
  # otherwise, yes
  return 1
}

### DCC COMMANDS ###
# Kills a partyline user.
bind dcc n kill dcc_kill
proc dcc_kill {hand idx arg} {
  if { $arg == "" } {
    putdcc $idx "Usage: kill <user> <reason>"
    return
  }
  if { [hand2idx [lindex $arg 0]] != -1 } {
    set tgt_handle [idx2hand [hand2idx [lindex $arg 0]]]
    putdcc [hand2idx [lindex $arg 0]] "Closing connection: Killed ($hand ([lreplace $arg 0 0]))"
    killdcc [hand2idx [lindex $arg 0]]
    dccbroadcast "$tgt_handle killed ($hand ([lreplace $arg 0 0]))"
  }
}

# Warns other +W fellas... same as IRC, except normal users can't see them.
# Period.
bind dcc - wallops dcc_wallops
proc dcc_wallops {hand idx arg} {
  if { $arg == "" } {
    putdcc $idx "Usage: wallops <message>"
    return
  }
  if { [matchattr $hand W] || [matchattr $hand t] } {
    wallops $hand $arg
    return
  }
  if { ![matchattr $hand W] && ![matchattr $hand t] } {
    putdcc $idx "Access denied."
    putloglev "c" "*" "\#$hand\# failed WALLOPS $arg"
    return
  }
}

# This proc actually performs the wallops... it's a separate proc so other
# procs might be able to use it too.
proc wallops {hand msg} {
  set wl_proc 0
  set wl_ulist [dcclist]
  putallbots "vBOTNET_WALLOPS $hand $msg"
  while { [lindex $wl_ulist $wl_proc] != "" } {
    if { [string tolower [lindex [lindex $wl_ulist $wl_proc] 3]] == "chat" && [validuser [lindex [lindex $wl_ulist $wl_proc] 1]] } {
      if { [matchattr [lindex [lindex $wl_ulist $wl_proc] 1] W] } {
        putdcc [lindex [lindex $wl_ulist $wl_proc] 0] "\[Wallops/$hand\] $msg"
      }
    }
    incr wl_proc
  }
}

# This one's to receive a WALLOPS message over the botnet and broadcast it to
# local +W users.
bind bot - vBOTNET_WALLOPS bot_wallops
proc bot_wallops {bot cmd arg} {
  set hand [lindex $arg 0]
  set msg [lreplace $arg 0 0]
  set wl_proc 0
  set wl_ulist [dcclist]
  while { [lindex $wl_ulist $wl_proc] != "" } {
    if { [string tolower [lindex [lindex $wl_ulist $wl_proc] 3]] == "chat" && [validuser [lindex [lindex $wl_ulist $wl_proc] 1]] } {
      if { [matchattr [lindex [lindex $wl_ulist $wl_proc] 1] W] } {
        putdcc [lindex [lindex $wl_ulist $wl_proc] 0] "\[Wallops/$hand@$bot\] $msg"
      }
    }
    incr wl_proc
  }
}

proc save_console_dcc {handle idx param} {
  set cons [console $idx]
  set chan [getchan $idx]
  user-set $handle conchan [lindex $cons 0]
  user-set $handle console [lindex $cons 1]
  user-set $handle echo [echo $idx]
  user-set $handle chan $chan
  set newcons "[lindex $cons 0]: [lindex $cons 1]"
  if {[echo $idx]} {
    putdcc $idx "Saved your console mode as $newcons (echo on) (chan $chan)"
  } {   
    putdcc $idx "Saved your console mode as $newcons (echo off) (chan $chan)"
  }
  return 1
}

# Does a rehash on your entire subnet. (useful if you have more than one bot
# sharing scripts that you're writing...like me. :)
bind dcc m srehash dcc_srehash
bind dcc m mrehash dcc_srehash
proc dcc_srehash {hand idx arg} {
  global bot_subnet
  putloglev "c" "*" "\#$hand\# SREHASH"
  putdcc $idx "Rehashing all bots on subnet..."
  if { $bot_subnet != "" } {
    putsubnet "vBOTNET_REHASH $bot_subnet $hand"
  }
  if { $bot_subnet == "" } {
    putsubnet "vBOTNET_REHASH NEW $hand"
  }
  rehash
}

bind bot - vBOTNET_REHASH bot_rehash
proc bot_rehash {bot cmd arg} {
  global bot_subnet warn-subnet-failures
  if { ![subnetvalid $bot $cmd $arg] } {
    putloglev "c" "*" "\#[lindex $arg 1]@$bot\# failed SREHASH (subnet cmds disabled)"
    if { ${warn-subnet-failures} == "yes" } {
      putbot $bot "vBOTNET_SUBNET_DISABLED $cmd"
    }
    return
  }
  if { [lindex $arg 0] == $bot_subnet } {
    putloglev "c" "*" "\#[lindex $arg 1]@$bot\# SREHASH"
    rehash
  }
}

# Same as srehash, but does a restart instead.
bind dcc m srestart dcc_srestart
proc dcc_srestart {hand idx arg} {
  global bot_subnet
  if { $bot_subnet == "" } {
    putloglev "c" "*" "\#$hand\# failed SRESTART (subnet cmds disabled)"
    putdcc $idx "Can't subnet restart -- subnet commands disabled."
    return
  }
  putloglev "c" "*" "\#$hand\# SRESTART"
  putdcc $idx "Restarting all bots on subnet..."
  if { $bot_subnet != "" } {
    putsubnet "vBOTNET_RESTART $bot_subnet $hand"
  }
  if { $bot_subnet == "" } {
    putsubnet "vBOTNET_RESTART NEW $hand"
  }
  restart
}

bind bot - vBOTNET_RESTART bot_restart
proc bot_restart {bot cmd arg} {
  global bot_subnet warn-subnet-failures
  if { ![subnetvalid $bot $cmd $arg] } {
    putloglev "c" "*" "\#[lindex $arg 1]@$bot\# failed SRESTART (subnet cmds disabled)"
    if { ${warn-subnet-failures} == "yes" } {
      putbot $bot "vBOTNET_SUBNET_DISABLED $cmd"
    }
    return
  }
  if { [lindex $arg 0] == $bot_subnet } {
    putloglev "c" "*" "\#[lindex $arg 1]@$bot\# SRESTART"
    restart
  }
}

# Asks all the bots whether or not they're running eggdrop extensions, and
# which version.
bind dcc m extcheck dcc_extcheck
proc dcc_extcheck {hand idx arg} {
  global bnnick eggext_version
  putdcc $idx "Checking for Eggdrop Extensions on botnet..."
  putdcc $idx "$bnnick is running Eggdrop Extensions version $eggext_version."
  putallbots "vBOTNET_CHECK_EGGDROP_EXT $idx $hand"
}

bind bot - vBOTNET_CHECK_EGGDROP_EXT bot_extcheck
proc bot_extcheck {bot cmd arg} {
  global eggext_version bnnick
  putloglev "c" "*" "\#[lindex $arg 1]@$bot\# EXTCHECK"
  putbot $bot "vBOTNET_CHECK_EGGDROP_EXT_ACK [lindex $arg 0] $bnnick running Eggdrop Extensions version $eggext_version"
}

bind bot - vBOTNET_CHECK_EGGDROP_EXT_ACK bot_extcheckack
proc bot_extcheckack {bot cmd arg} {
  putdcc [lindex $arg 0] "$bot acknowledged: [lreplace $arg 0 0]"
}

# Ping a bot! Doesn't measure lag time, but DOES check if the bot is still
# alive.
bind dcc - pingbot dcc_pingbot
proc dcc_pingbot {hand idx arg} {
  if { $arg == "" } {
    putdcc $idx "Usage: pingbot <bot>"
    return
  }
  putdcc $idx "Pinging bot [lindex $arg 0]..."
  if { [lsearch [string tolower [bots]] [string tolower [lindex $arg 0]]] == "-1" } {
    putdcc $idx "ERROR: Bot not found."
    return
  }
  putbot [lindex $arg 0] "vBOTNET_PING $idx $hand"
}

# ping/pong- a simple tool to let you measure lag time across a botnet (to
# some degree anyway -- this doesn't measure time yet), and to check if
# another bot is dead or not
bind bot - vBOTNET_PING bot_ping
proc bot_ping {bot cmd arg} {
  putloglev "c" "*" "\#[lindex $arg 1]@$bot\# PING"
  putbot $bot "vBOTNET_PONG [lindex $arg 0]"
}

bind bot - vBOTNET_PONG bot_pong
proc bot_pong {bot cmd arg} {
  putdcc [lindex $arg 0] "Received PING reply from $bot"
}

# version of wallops is sent to everyone on the botnet, available to +t or
# higher
bind dcc t wall dcc_wall
proc dcc_wall {hand idx arg} {
  if { $arg == "" } {
    putdcc $idx "Usage: wall <message>"
    return
  }
  dccbroadcast "\[Wall/$hand\] $arg"
}

# Changes or checks the bot's nick.
bind dcc m botnick dcc_botnick
proc dcc_botnick {hand idx arg} {
  global nick altnick botnick
  if { $arg == "" } {
      putdcc $idx "Usage: botnick <nick> \[altnick\]"
      putdcc $idx "Bot's IRC nick is $botnick ::: Primary nick: $nick ::: Secondary nick: $altnick"
    return
  }
  set nick [lindex $arg 0]
  putdcc $idx "Set primary bot nick to [lindex $arg 0]"
  if { [llength $arg] == 2 } {
    set altnick [lindex $arg 1]
    putdcc $idx "Set secondary bot nick to [lindex $arg 1]"
  }
  if { [llength $arg] == 1 } {
    set altnick "[string range [lindex $arg 0] 0 7]_"
    putdcc $idx "Set secondary bot nick to [string range [lindex $arg 0] 0 7]_"
  }
  putloglev "c" "*" "\#$hand\# BOTNICK $arg"
}

bind dcc m mjoin dcc_mjoin
proc dcc_mjoin {hand idx arg} {
    global bot_subnet
    if { $arg == "" } {
	putdcc $idx "Usage: mjoin <channel>"
	return
    }
    if { $bot_subnet == "" } {
      putloglev "c" "*" "\#$hand\# failed MJOIN $arg (subnet cmds disabled)"
      putdcc $idx "Can't mass join -- subnet commands disabled."
      return
    }
    if { [vldchan [lindex $arg 0]] } {
        putserv "JOIN [lindex $arg 0]"
        return
    }
    channel add [lindex $arg 0]
    channel set [lindex $arg 0] -clearbans -enforcebans -dynamicbans +userbans -autoop -bitch -greet +protectops +statuslog -stopnethack -revenge -autovoice -secret -shared
    putdcc $idx "Joining channel [lindex $arg 0] ..."
    if { $bot_subnet == "" } {
        putsubnet "vBOTNET_MJOIN NEW $arg"
    }
    if { $bot_subnet != "" } {
        putsubnet "vBOTNET_MJOIN $bot_subnet $arg"
    }
    putloglev "c" "*" "\#$hand\# MJOIN $arg"
}

bind bot - vBOTNET_MJOIN botnet_mjoin
proc botnet_mjoin {bot cmd arg} {
    global bot_subnet warn-subnet-failures
    if { ![subnetvalid $bot $cmd $arg] } {
      putloglev "c" "*" "\#bot:$bot\# failed MJOIN $arg (subnet cmds disabled)"
      if { ${warn-subnet-failures} == "yes" } {
        putbot $bot "vBOTNET_SUBNET_DISABLED $cmd"
      }
      return
    }
    if { $bot_subnet != [lindex $arg 0] } {
        return
    }
    if { [vldchan [lindex $arg 1]] } {
        putserv "JOIN [lindex $arg 1]"
        return
    }
    channel add [lindex $arg 1]
    channel set [lindex $arg 1] -clearbans -enforcebans -dynamicbans +userbans -autoop -bitch -greet +protectops +statuslog -stopnethack -revenge -autovoice -secret -shared
    putloglev "c" "*" "\#bot:$bot\# MJOIN $arg"
}

# Makes the bot join a channel... a simplified version of +chan, it makes the
# bot join the channel, but do nothing there.
bind dcc m join dcc_cmd_join
proc dcc_cmd_join {hand idx arg} {
    if { $arg == "" } {
	putdcc $idx "Usage: join <channel>"
	return
    }
    if { [vldchan [lindex $arg 0]] } {
        putserv "JOIN [lindex $arg 0]"
        return
    }
    channel add [lindex $arg 0]
    channel set [lindex $arg 0] -clearbans -enforcebans -dynamicbans +userbans -autoop -bitch -greet +protectops +statuslog -stopnethack -revenge -autovoice -secret -shared
    putdcc $idx "Joining channel [lindex $arg 0] ..."
    putloglev "c" "*" "\#$hand\# JOIN $arg"
}

# Same as join, except that it sets your console to the channel and sets +p
# on the console, so it's like your bot is your own client. (kinda..)
bind dcc m cjoin dcc_cmd_cjoin
proc dcc_cmd_cjoin {hand idx arg} {
    if { $arg == "" } {
	putdcc $idx "Usage: cjoin <channel>"
	return
    }
    if { [vldchan [lindex $arg 0]] } {
        putserv "JOIN [lindex $arg 0]"
        return
    }
    channel add [lindex $arg 0]
    channel set [lindex $arg 0] -clearbans -enforcebans -dynamicbans +userbans -autoop -bitch -greet +protectops +statuslog -stopnethack -revenge -autovoice -secret -shared
    console $idx [lindex $arg 0] +p
    putdcc $idx "Joining channel [lindex $arg 0] ..."
    putloglev "c" "*" "\#$hand\# CJOIN $arg"
}

# Leaves the channel.
bind dcc m part dcc_cmd_part
proc dcc_cmd_part {hand idx arg} {
    if { $arg == "" } {
	putdcc $idx "Usage: part <channel>"
	return
    }
    if { ![vldchan [lindex $arg 0]] } {
        putdcc $idx "Invalid channel: [lindex $arg 0]"
        return
    }
    channel remove [lindex $arg 0]
    putdcc $idx "Leaving channel [lindex $arg 0] ..."
    putloglev "c" $arg "\#$hand\# PART $arg"
}

bind dcc m mpart dcc_mpart
proc dcc_mpart {hand idx arg} {
    global bot_subnet
    if { $arg == "" } {
	putdcc $idx "Usage: mpart <channel>"
	return
    }
    if { $bot_subnet == "" } {
      putloglev "c" "*" "\#$hand\# failed MPART $arg (subnet cmds disabled)"
      putdcc $idx "Can't mass part -- subnet commands disabled."
      return
    }
    if { [vldchan [lindex $arg 0]] } {
        putserv "PART [lindex $arg 0]"
        return
    }
    channel remove [lindex $arg 0]
    putdcc $idx "Leaving channel [lindex $arg 0] ..."
    if { $bot_subnet == "" } {
        putsubnet "vBOTNET_MPART NEW $arg"
    }
    if { $bot_subnet != "" } {
        putsubnet "vBOTNET_MPART $bot_subnet $arg"
    }
    putloglev "c" "*" "\#$hand\# MPART $arg"
}

bind bot - vBOTNET_MPART botnet_mpart
proc botnet_mpart {bot cmd arg} {
    global bot_subnet warn-subnet-failures
    if { ![subnetvalid $bot $cmd $arg] } {
      putloglev "c" "*" "\#bot:$bot\# failed MPART $arg (subnet cmds disabled)"
      if { ${warn-subnet-failures} == "yes" } {
        putbot $bot "vBOTNET_SUBNET_DISABLED $cmd"
      }
      return
    }
    if { $bot_subnet != [lindex $arg 0] } {
        return
    }
    if { [vldchan [lindex $arg 1]] } {
        putserv "PART [lindex $arg 1]"
        return
    }
    channel remove [lindex $arg 1]
    putloglev "c" "*" "\#bot:$bot\# MPART $arg"
}

# A DCC interface to restore_console, it restores your saved console.
bind dcc t restore dcc_cmd_restore
proc dcc_cmd_restore {hand idx arg} {
  putdcc $idx "Restoring your saved console state..."
  restore_console $hand $idx
}

# Take over a channel quickly and efficiently.
bind dcc m takeover dcc_cmd_takeover
proc dcc_cmd_takeover {hand idx arg} {
  global autoban_channels botnick
  if { $arg == "" } {
    putdcc $idx "Usage: takeover <channel> \[reason\]"
    putdcc $idx "Use with extreme caution!"
    return
  }
  if { ![vldchan [lindex $arg 0]] } {
    putdcc $idx "Invalid channel: [lindex $arg 0]"
    return
  }
  if { ![botisop [lindex $arg 0]] } {
    putdcc $idx "I don't have ops on [lindex $arg 0], can't take over."
    return
  }
  putdcc $idx "Taking over: [lindex $arg 0]"
  channel set [lindex $arg 0] chanmode +ntims
  channel set [lindex $arg 0] +enforcebans +bitch +stopnethack -autovoice -autoop -greet +revenge +secret
  lappend autoban_channels [lindex $arg 0]
  foreach tgt [chanlist [lindex $arg 0]] {
    if { [isop $tgt [lindex $arg 0]] && ![matchattr [nick2hand $tgt [lindex $arg 0]] f] && ![matchchanattr [nick2hand $tgt [lindex $arg 0]] f [lindex $arg 0]] && $tgt != $botnick } {
      pushmode [lindex $arg 0] -o $tgt
    }
    flushmode [lindex $arg 0]
  }
  pushmode [lindex $arg 0] +n
  pushmode [lindex $arg 0] +t
  pushmode [lindex $arg 0] +i
  pushmode [lindex $arg 0] +m
  pushmode [lindex $arg 0] +s
  pushmode [lindex $arg 0] +k `GoNE`
  flushmode [lindex $arg 0]
  foreach tgt [chanlist [lindex $arg 0]] {
    if { ![matchattr [nick2hand $tgt [lindex $arg 0]] f] && ![matchchanattr [nick2hand $tgt [lindex $arg 0]] f [lindex $arg 0]] && $tgt != $botnick } {
      putserv "KICK [lindex $arg 0] $tgt"
    }
  }
  set reason [lreplace $arg 0 0]
  if { $reason == "" } {
    set reason "This channel is LAME!!!"
  }
  putserv "TOPIC [lindex $arg 0] :TakeoveR: $reason"
  putloglev "c" $arg "\#$hand\# TAKEOVER $arg"
}

# Stop the effects of a takeover on a channel.
bind dcc m untakeover dcc_cmd_untakeover
proc dcc_cmd_untakeover {hand idx arg} {
  global autoban_channels
  if { $arg == "" } {
    putdcc $idx "Usage: untakeover <channel>"
    return
  }
  if { ![vldchan [lindex $arg 0]] } {
    putdcc $idx "I'm not on that channel!"
    return
  }
  pushmode [lindex $arg 0] -t
  pushmode [lindex $arg 0] -i
  pushmode [lindex $arg 0] -m
  pushmode [lindex $arg 0] -s
  pushmode [lindex $arg 0] -k `GoNE`
  flushmode [lindex $arg 0]
  putserv "TOPIC [lindex $arg 0] :UnTakeoveR Finished."
  channel remove [lindex $arg 0]
  if { [lsearch $autoban_channels [lindex $arg 0]] != -1 } {
    set autoban_channels [lreplace $autoban_channels [lsearch $autoban_channels [lindex $arg 0]] [lsearch $autoban_channels [lindex $arg 0]]]
  }
  putloglev "c" $arg "\#$hand\# UNTAKEOVER $arg"
}

# Sorta like the who /msg command, only on DCC.
bind dcc - whochan dcc_cmd_whochan
proc dcc_cmd_whochan {hand idx arg} {
  if { $arg == "" } {
    putdcc $idx "Usage: whochan <channel>"
    return
  }
  if { ![vldchan [lindex $arg 0]] } {
    putdcc $idx "Invalid channel: [lindex $arg 0]"
    return
  }
  if { ![matchattr $hand o] && ![matchchanattr $hand o $chan] } {
    putdcc $idx "Access denied: must be +o on channel $chan"
    return
  }
  putdcc $idx "Users on [lindex $arg 0]: [chanlist [lindex $arg 0]]"
}

### DCC TRIGGERS ###
# This proc is executed when someone joins the DCC partyline.
bind chon - ** dcc_chon
proc dcc_chon {hand idx} {
  global bnnick botnick internal-store
  # There used to be K-line stuff here... I scrapped it, because it's useless.
  # Do a +ignore if you feel there's a host trying to connect that shouldn't.
  if { [user-get $hand exp] != 1 } {
    # Check if the user that logged in has before... if not, give this cute
    # little message :)
    user-set $hand exp "1"
    putdcc $idx "---> Welcome to the eggdrop partyline!  Please read what follows..."
    putdcc $idx "You will have a few commands you will need to learn."
    putdcc $idx ".help    : Gets you help on your commands.  The most important."
    putdcc $idx ".quit    : Quits the partyline."
    putdcc $idx ".whom    : Shows you who's here."
    putdcc $idx ".me      : Like a IRC /me command."
    putdcc $idx ".info    : Changes your info line, which $botnick says when you join a channel $botnick is on."
    putdcc $idx ".chpass  : Changes your password."
    putdcc $idx ".bottree : Shows a tree of the other bots that are on the network."
    putdcc $idx "Everything else is normal chat and is seen by everyone."
  }
  if {${internal-store} != "yes"} {
    # Restore the console
    restore_console $hand $idx
  }
  # If they're +t or higher, give 'em WALLOPS access.
  if { [matchattr $hand t] == 1 && [matchattr $hand W] != 1 } {
    chattr $hand +W
  }
}

### CHANNEL/MSG TRIGGERS ###
# Procedure executed when someone joins a channel.
bind join - ** ch_join
proc ch_join {nick host hand chan} {
  global botnick auto_id autoban_channels bnnick
    # The open hellos stuff has been removed because it's a serious security
    # compromise. Deal with it.
  # Check if this is an autoban channel, and ban this mofo if they're not +f
  if { [lsearch $autoban_channels $chan] != -1 && $nick != $botnick } {
    if { ![matchattr $hand f] && ![matchchanattr $hand f $chan] } {
      newchanban $chan "$host" $bnnick "AutoBan!"
      putloglev "cj" $chan "($host) !$hand! JOIN $chan (AutoBan!)"
    }
  }
  # Check for auto_id and auto identify people if the channel is set to +greet
  if { [lsearch [channel info $chan] "+greet"] != "-1" } {
    if { $auto_id != "no" } {
      pub_identify $botnick *@* $botnick $chan $nick
    }
  }
  # If a user doesn't have a set password, demand that they set one!
  if { [passwdok $hand ""] } {
    putserv "NOTICE $nick :You don't have a password set on $botnick! Please set it now: /msg $botnick PASS <password>"
  }
  # Check if they're logged in... if they are, and not on any of the bot's
  # covered channels, EXCLUDING the one they just joined, log 'em out.
  if { [getuser $hand xtra login-host] != "" } {
    set logemout 1
    # make sure that they're logged in under the exact same nick!user@host
    # as the person who joined...
    if { [getuser $hand xtra login-host] == "$nick!$host" } {
      foreach tgt [channels] {
        # please make sure that we don't think they already were on the channel
        # they just joined...
        if { $tgt != $chan } {
          if { [onchan $nick $tgt] } {
            set logemout 0
          }
        }
      }
    }
    # Well, if we're going to log 'em out after all that testing, lez do it..
    if { $logemout == 1 } {
      setuser $hand xtra login-host ""
    }
  }
}

if {${internal-store} != "yes"} {
  if {${console-autosave}} {
    bind chof - * save_console
  } {
    bind dcc - store save_console_dcc
  }
}

### CHANNEL/MSG COMMANDS ###
# Obsolete .topic channel command: insecure
# Obsolete OS command: Linux-only, not cross platform.

bind pub - .kiss pub_kiss
proc pub_kiss {nick host hand chan arg} {
  global botnick
  set lover $arg
  putserv "PRIVMSG $chan :ACTION leans over and gives $lover a big kiss :)"
  putloglev "c" $chan "($host) !$hand! $chan KISS $arg"
}

# Obsolete voice/kick/ban/op/mode commands: insecure.

# The identify commands match a user on a channel to someone in the userfile.
# The reverse identify does the opposite, searching for a user in the channel
# with a handle in the userfile. It can be used in DCC, on the channel, and
# MSG.
bind pub - .identify pub_identify
bind pub - .id pub_identify
proc pub_identify {nick host hand chan arg} {
  global botnick
  if { $arg == "" } {
    putserv "PRIVMSG $chan :Usage: .identify <nickname>"
    return
  }
  putloglev "c" $chan "($host) !$hand! $chan IDENTIFY $arg"
  if { [onchan [lindex $arg 0] $chan] == "0" } {
    putserv "PRIVMSG $chan :IDENTIFY: [lindex $arg 0] is not on the channel."
    return
  }
  if { [string tolower [lindex $arg 0]] == [string tolower $botnick] } {
    putserv "PRIVMSG $chan :IDENTIFY: That's me!"
    return
  }
  set tgt_handle [nick2hand [lindex $arg 0] $chan]
  if { $tgt_handle == "*" } {
    putserv "PRIVMSG $chan :IDENTIFY: Cannot identify [lindex $arg 0]!"
    return
  }
  putserv "PRIVMSG $chan :IDENTIFY: Identified [lindex $arg 0] as $tgt_handle."
  return
}

bind pub - .rid pub_ridentify
bind pub - .ridentify pub_ridentify
proc pub_ridentify {nick host hand chan arg} {
  global botnick
  if { $arg == "" } {
    putserv "PRIVMSG $chan :Usage: .ridentify <handle>"
    return
  }
  putloglev "c" $chan "($host) !$hand! $chan RIDENTIFY $arg"
  if { [string tolower [lindex $arg 0]] == [string tolower $botnick] } {
    putserv "PRIVMSG $chan :IDENTIFY: That's me!"
    return
  }
  set tgt_handle [hand2nick $arg $chan]
  if { $tgt_handle == "" } {
    putserv "PRIVMSG $chan :IDENTIFY: Cannot reverse identify [lindex $arg 0]!"
    return
  }
  putserv "PRIVMSG $chan :IDENTIFY: Reverse identified [lindex $arg 0] as $tgt_handle."
  return
}

bind dcc - identify dcc_identify
bind dcc - id dcc_identify
proc dcc_identify {hand idx arg} {
  global botnick
  if { $arg == "" } {
    putdcc $idx "Usage: identify <nickname>"
    return
  }
  if { [lindex $arg 1] == "" } {
    set rchan [lindex [console $idx] 0]
  }
  if { [lindex $arg 1] != "" } {
    set rchan [lindex $arg 1]
  }
  putloglev "c" $rchan "\#$hand\# IDENTIFY $arg"
  if { [onchan [lindex $arg 0] $rchan] == "0" } {
    putdcc $idx "IDENTIFY: [lindex $arg 0] is not on the channel $rchan."
    return
  }
  if { [lindex $arg 0] == $botnick } {
    putdcc $idx "IDENTIFY: That's me!"
    return
  }
  set tgt_handle [nick2hand [lindex $arg 0] $rchan]
  if { $tgt_handle == "*" } {
    putdcc $idx "IDENTIFY: Cannot identify [lindex $arg 0] on $rchan!"
    return
  }
  putdcc $idx "IDENTIFY: Identified [lindex $arg 0] on $rchan as $tgt_handle."
  return
}

bind dcc - rid dcc_ridentify
bind dcc - ridentify dcc_ridentify
proc dcc_ridentify {hand idx arg} {
  global botnick
  if { $arg == "" } {
    putdcc $idx "Usage: .ridentify <handle>"
    return
  }
  if { [lindex $arg 1] == "" } {
    set rchan [lindex [console $idx] 0]
  }
  if { [lindex $arg 1] != "" } {
    set rchan [lindex $arg 1]
  }
  putloglev "c" $rchan "\#$hand\# $rchan RIDENTIFY $arg"
  if { [string tolower [lindex $arg 0]] == [string tolower $botnick] } {
    putdcc $idx "IDENTIFY: That's me!"
    return
  }
  set tgt_handle [hand2nick $arg $rchan]
  if { $tgt_handle == "" } {
    putdcc $idx "IDENTIFY: Cannot reverse identify [lindex $arg 0]!"
    return
  }
  putdcc $idx "IDENTIFY: Reverse identified [lindex $arg 0] as $tgt_handle."
  return
}

bind msg - identify msg_identify
bind msg - id msg_identify
bind msg - .id msg_identify
bind msg - .identify msg_identify
proc msg_identify {nick host hand arg} {
  global botnick
  if { $arg == "" } {
    putserv "PRIVMSG $nick :Usage: identify <nickname>"
    return 255
  }
  if { [lindex $arg 1] == "" } {
    putserv "PRIVMSG $nick :Usage: identify <nickname>"
    return 255
  }
  if { [lindex $arg 1] != "" } {
    set rchan [lindex $arg 1]
  }
  putloglev "c" $rchan "[$nick!host] !$hand! IDENTIFY $arg"
  if { [onchan [lindex $arg 0] $rchan] == "0" } {
    putserv "PRIVMSG $nick :IDENTIFY: [lindex $arg 0] is not on the channel $rchan."
    return 128
  }
  if { [lindex $arg 0] == $botnick } {
    putserv "PRIVMSG $nick :IDENTIFY: That's me!"
    return 0
  }
  set tgt_handle [nick2hand [lindex $arg 0] $rchan]
  if { $tgt_handle == "*" } {
    putserv "PRIVMSG $nick :IDENTIFY: Cannot identify [lindex $arg 0] on $rchan!"
    return 129
  }
  putserv "PRIVMSG $nick :IDENTIFY: Identified [lindex $arg 0] on $rchan as $tgt_handle."
  return 0
}

# Does a cute #WareZZeraW ad. :)
bind pub - .warezad pub_wzad
proc pub_wzad {nick host hand chan arg} {
  putloglev "c" $chan "($host) !$hand! $chan WAREZAD"
  putserv "PRIVMSG $chan :- \[ # 4w 7a 13r 12e z .12 z e r a w \] -"
}

# The bot will tell you your userflags, and flags on the channel.
bind pub - .auth pub_auth
proc pub_auth {nick host hand chan arg} {
  if { $arg == "" } {
    putserv "NOTICE $nick :Usage: .auth <user>"
    return
  }
  if { ![validuser $arg] } {
    putserv "PRIVMSG $chan :Auth Error: Handle does not exist."
    return
  }
  putserv "PRIVMSG $chan :Auth: User $arg [chattr $arg $chan] $chan."
  putloglev "c" $chan "($nick!$host) !$hand! $chan AUTH $arg"
  return
}

# This modified version of .auth works in /MSG.
bind msg - .auth msg_auth
bind msg - auth msg_auth
proc auth {nick host hand arg} {
  if { $arg == "" } {
    putserv "NOTICE $nick :Usage: .auth"
    return
  }
  if { ![validuser $hand] } {
    putserv "PRIVMSG $chan :Auth Error: You are not a registered user."
    return
  }
  putserv "PRIVMSG $chan :Auth: You are +[chattr [nick2hand $arg $chan]]."
  putloglev "c" "*" "\[$nick!$host\] !$hand! AUTH $arg"
  return
}

# Last seen stuff.
bind pub - .seen pub_seen
proc pub_seen {nick host hand chan arg} {
  global timezone
  if { $arg == "" } {
    putserv "PRIVMSG $chan :Usage: .seen <user1> <user2>..."
    return
  }
  putloglev "c" $chan "($host) !$hand! $chan SEEN $arg"
  foreach tgt $arg {
    if { ![validuser $tgt] } {
      putserv "PRIVMSG $chan :Unregistered user: $tgt"
    }
    if { [validuser $tgt] } {
      set seen-data [getuser $tgt laston]
      putserv "PRIVMSG $chan :Last saw $tgt at [ctime [lindex ${seen-data} 0]] $timezone on [lindex ${seen-data} 1]."
    }
  }
}

bind msg - .seen msg_seen
bind msg - seen msg_seen
proc msg_seen {nick host hand arg} {
  global timezone
  if { $arg == "" } {
    putserv "PRIVMSG $chan :Usage: .seen <user1> <user2>..."
    return
  }
  putloglev "c" "*" "\[$nick!$host\] !$hand! SEEN $arg"
  foreach tgt $arg {
    if { ![validuser $tgt] } {
      putserv "PRIVMSG $chan :Unregistered user: $tgt"
    }
    if { [validuser $tgt] } {
      set seen-data [getuser $tgt laston]
      putserv "NOTICE $nick :Last saw $tgt at [ctime [lindex ${seen-data} 0]] $timezone on [lindex ${seen-data} 1]."
    }
  }
}

bind dcc - ctime dcc_ctime
proc dcc_ctime {hand idx arg} {
  if { $arg == "" } {
    putdcc $idx "Usage: ctime <unixtime>"
    return
  }
  putdcc $idx "[ctime $arg]"
}

bind pub - .ctime pub_ctime
proc pub_ctime {nick host hand chan arg} {
  if { $arg == "" } {
    putserv "PRIVMSG $chan :Usage: ctime <unixtime>"
    return
  }
  putserv "PRIVMSG $chan :[ctime $arg]"
  putloglev "c" $chan "($host) !$hand! $chan CTIME $arg"
}

bind msg - .ctime msg_ctime
bind msg - ctime msg_ctime
proc msg_ctime {nick host hand arg} {
  if { $arg == "" } {
    putserv "NOTICE $nick :Usage: ctime <unixtime>"
    return
  }
  putserv "NOTICE $nick :[ctime $arg]"
  putloglev "c" "*" "\[$nick!$host\] !$hand! CTIME $arg"
}

bind msg - .login msg_login
bind msg - login msg_login
proc msg_login {nick host hand arg} {
  global quiet-reject
  if { ![validuser $hand] } {
    putloglev "c" "*" "\[$nick!$host\] !$hand! failed LOGIN (not in userfile)"
    if { quiet-reject != 1 } {
      putserv "NOTICE $nick :You are not a registered user."
    }
    return
  }
  if { $arg == "" } {
    putserv "NOTICE $nick :Usage: login <password>"
    return
  }
  if { ![passwdok $hand $arg] } {
    putloglev "c" "*" "\[$nick!$host\] !$hand! failed LOGIN (password incorrect)"
    putserv "NOTICE $nick :Password incorrect, login failed."
    return
  }
  putloglev "c" "*" "\[$nick!$host\] !$hand! LOGIN \[something\]"
  putserv "NOTICE $nick :Password correct, login successful."
  if { [lsearch $host "$nick!*"] == -1 } {
    setuser $hand xtra login-host "$nick!$host"
  }
  if { [lsearch $host "$nick!*"] != -1 } {
    setuser $hand xtra login-host $host
  }
}

bind msg - .logout msg_logout
bind msg - logout msg_logout
bind msg - .logoff msg_logout
bind msg - logoff msg_logout
proc msg_logout {nick host hand arg} {
  global quiet-reject
  if { ![validuser $hand] } {
    putloglev "c" "*" "\[$nick!$host\] !$hand! failed LOGOUT (not in userfile)"
    if { quiet-reject != 1 } {
      putserv "NOTICE $nick :You are not a registered user."
    }
    return
  }
  if { $arg == "" } {
    putserv "NOTICE $nick :Usage: logout <password>"
    return
  }
  if { ![isloggedin $hand "$nick!$host"] } {
    putloglev "c" "*" "\[$nick!$host\] !$hand! failed LOGOUT (not logged in)"
    putserv "NOTICE $nick :You are not logged in."
    return
  }
  if { ![passwdok $hand $arg] } {
    putloglev "c" "*" "\[$nick!$host\] !$hand! failed LOGOUT (password incorrect)"
    putserv "NOTICE $nick :Password incorrect."
    return
  }
  putloglev "c" "*" "\[$nick!$host\] !$hand! LOGOUT \[something\]"
  setuser $hand xtra login-host ""
  putserv "NOTICE $nick :Logged out."
}

bind pub o|o .op pub_op
proc pub_op {nick host hand chan arg} {
  global botnick
  if { ![botisop $chan] } {
    putserv "PRIVMSG $chan :Access denied: I am not opped on $chan."
    putloglev "c" $chan "($host) !$hand! $chan failed OP $arg (bot not opped)"
    return
  }
  if { ![isloggedin $hand "$nick!$host"] } {
    putloglev "c" $chan "($host) !$hand! $chan failed OP $arg (not logged in)"
    putserv "PRIVMSG $chan :Access denied: $nick is not logged in."
    return
  }
  if { $arg == "" } {
    putserv "PRIVMSG $chan :Usage: .op <nick1> <nickN>..."
    return
  }
  putloglev "c" $chan "($host) !$hand! $chan OP $arg"
  foreach tgt $arg {
    pushmode $chan +o $tgt
  }
}

bind pub o|o .deop pub_deop
bind pub o|o .dop pub_deop
proc pub_deop {nick host hand chan arg} {
  global botnick
  if { ![botisop $chan] } {
    putserv "PRIVMSG $chan :Access denied: I am not opped on $chan."
    putloglev "c" $chan "($host) !$hand! $chan failed DEOP $arg (bot not opped)"
    return
  }
  if { ![isloggedin $hand "$nick!$host"] } {
    putloglev "c" $chan "($host) !$hand! $chan failed DEOP $arg (not logged in)"
    putserv "PRIVMSG $chan :Access denied: $nick is not logged in."
    return
  }
  if { $arg == "" } {
    putserv "PRIVMSG $chan :Usage: .deop <nick1> <nickN>..."
    return
  }
  putloglev "c" $chan "($host) !$hand! $chan DEOP $arg"
  foreach tgt $arg {
    pushmode $chan -o $tgt
  }
}

bind pub o|o .voice pub_voice
proc pub_voice {nick host hand chan arg} {
  global botnick
  if { ![botisop $chan] } {
    putserv "PRIVMSG $chan :Access denied: I am not opped on $chan."
    putloglev "c" $chan "($host) !$hand! $chan failed VOICE $arg (bot not opped)"
    return
  }
  if { ![isloggedin $hand "$nick!$host"] } {
    putloglev "c" $chan "($host) !$hand! $chan failed VOICE $arg (not logged in)"
    putserv "PRIVMSG $chan :Access denied: $nick is not logged in."
    return
  }
  if { $arg == "" } {
    putserv "PRIVMSG $chan :Usage: .voice <nick1> <nickN>..."
    return
  }
  putloglev "c" $chan "($host) !$hand! $chan VOICE $arg"
  foreach tgt $arg {
    pushmode $chan +v $tgt
  }
}

bind pub o|o .devoice pub_devoice
proc pub_devoice {nick host hand chan arg} {
  global botnick
  if { ![botisop $chan] } {
    putserv "PRIVMSG $chan :Access denied: I am not opped on $chan."
    putloglev "c" $chan "($host) !$hand! $chan failed DEVOICE $arg (bot not opped)"
    return
  }
  if { ![isloggedin $hand "$nick!$host"] } {
    putloglev "c" $chan "($host) !$hand! $chan failed DEVOICE $arg (not logged in)"
    putserv "PRIVMSG $chan :Access denied: $nick is not logged in."
    return
  }
  if { $arg == "" } {
    putserv "PRIVMSG $chan :Usage: .devoice <nick1> <nickN>..."
    return
  }
  putloglev "c" $chan "($host) !$hand! $chan DEVOICE $arg"
  foreach tgt $arg {
    pushmode $chan -v $tgt
  }
}

bind pub o|o .kick pub_kick
proc pub_kick {nick host hand chan arg} {
  global botnick
  if { ![botisop $chan] } {
    putserv "PRIVMSG $chan :Access denied: I am not opped on $chan."
    putloglev "c" $chan "($host) !$hand! $chan failed KICK $arg (bot not opped)"
    return
  }
  if { ![isloggedin $hand "$nick!$host"] } {
    putloglev "c" $chan "($host) !$hand! $chan failed KICK $arg (not logged in)"
    putserv "PRIVMSG $chan :Access denied: $nick is not logged in."
    return
  }
  if { $arg == "" } {
    putserv "PRIVMSG $chan :Usage: .kick <nick1> <nickN>..."
    return
  }
  putloglev "c" $chan "($host) !$hand! $chan KICK $arg"
  foreach tgt $arg {
    putserv "KICK $chan $tgt :Requested by $host ($hand)"
  }
}

bind dcc m msay dcc_msay
proc dcc_msay {hand idx arg} {
  global bot_subnet
  set str $arg
  if { ![vldchan [lindex $str 0]] } {
    set chan [lindex [console $idx] 0]
  }
  if { [vldchan [lindex $str 0]] } {
    set chan [lindex $str 0]
    set str [lreplace $str 0 0]
  }
  if { ![vldchan $chan] } {
    putdcc $idx "ERROR: invalid channel $chan"
    putdcc $idx "Continuing with mass say..."
  }
  if { $bot_subnet == "" } {
    putloglev "c" $chan "\#$hand\# failed MSAY $str (subnet cmds disabled)"
    putdcc $idx "Can't mass say -- subnet commands disabled."
    return
  }
  if { [vldchan $chan] } {
    putserv "PRIVMSG $chan :$str"
  }
  if { $bot_subnet == "" } {
    putsubnet "vBOTNET_MSAY NEW $chan $str"
  }
  if { $bot_subnet != "" } {
    putsubnet "vBOTNET_MSAY $bot_subnet $chan $str"
  }
  putloglev "c" $chan "\#$hand\# MSAY $str"
}

bind bot - vBOTNET_MSAY botnet_msay
proc botnet_msay {bot cmd arg} {
  global bot_subnet warn-subnet-failures
  if { [lindex $arg 0] == $bot_subnet } {
    if { [vldchan [lindex $arg 1]] } {
      if { ![subnetvalid $bot $cmd $arg] } {
        putloglev "c" [lindex $arg 1] "\#bot:$bot\# failed MSAY [lreplace $arg 0 1] (subnet cmds disabled)"
        if { ${warn-subnet-failures} == "yes" } {
          putbot $bot "vBOTNET_SUBNET_DISABLED $cmd"
        }
        return
      }
      putserv "PRIVMSG [lindex $arg 1] :[lreplace $arg 0 1]"
      putloglev "c" [lindex $arg 1] "\#bot:$bot\# MSAY [lreplace $arg 0 1]"
    }
  }
}

bind pub - .say pub_say
proc pub_say {nick host hand chan arg} {
  if { ![matchattr $hand m] && ![matchchanattr $hand m $chan] } {
    putserv "PRIVMSG $chan :Access denied."
    putloglev "c" $chan "($host) !$hand! $chan failed SAY $arg"
    return
  }
  if { ![isloggedin $hand "$nick!$host"] } {
    putserv "PRIVMSG $chan :Access denied - not logged in."
    putloglev "c" $chan "($host) !$hand! $chan failed SAY $arg"
    return
  }
  putloglev "c" $chan "($host) !$hand! $chan SAY $arg"
  putserv "PRIVMSG $chan :$arg"
}

bind pub - .act pub_act
proc pub_act {nick host hand chan arg} {
  if { ![matchattr $hand m] && ![matchchanattr $hand m $chan] } {
    putserv "PRIVMSG $chan :Access denied."
    putloglev "c" $chan "($host) !$hand! $chan failed ACT $arg"
    return
  }
  if { ![isloggedin $hand "$nick!$host"] } {
    putserv "PRIVMSG $chan :Access denied - not logged in."
    putloglev "c" $chan "($host) !$hand! $chan failed ACT $arg"
    return
  }
  putloglev "c" $chan "($host) !$hand! $chan ACT $arg"
  putserv "PRIVMSG $chan :ACTION $arg"
}

bind pub - .time pub_time
proc pub_time {nick host hand chan arg} {
  global timezone
  putserv "PRIVMSG $chan :Local time is: [ctime [unixtime]] $timezone"
}

bind msg - .time msg_time
bind msg - time msg_time
proc msg_time {nick host hand arg} {
  global timezone
  putserv "NOTICE $nick :Local time is: [ctime [unixtime]] $timezone"
}

bind pub - .addvoice pub_addvoice
proc pub_addvoice {nick host hand chan arg} {
  if { ![matchattr $hand o] && ![matchchanattr $hand o $chan] } {
    putserv "PRIVMSG $chan :Access denied."
    return
  }
  if { ![isloggedin $hand "$nick!$host"] } {
    putserv "PRIVMSG $chan :Access denied: not logged in."
    return
  }
  if { $arg == "" } {
    putserv "PRIVMSG $chan :Usage: addvoice <handle>"
    return
  }
  if { ![validuser $arg] } {
    putserv "PRIVMSG $chan :Error: Invalid handle."
    return
  }
  chattr $arg |+v $chan
  putserv "PRIVMSG $chan :Added $arg as voice on $chan."
}

bind pub - .addop pub_addop
proc pub_addop {nick host hand chan arg} {
  if { ![matchattr $hand o] && ![matchchanattr $hand m $chan] } {
    putserv "PRIVMSG $chan :Access denied."
    return
  }
  if { ![isloggedin $hand "$nick!$host"] } {
    putserv "PRIVMSG $chan :Access denied: not logged in."
    return
  }
  if { $arg == "" } {
    putserv "PRIVMSG $chan :Usage: addop <handle>"
    return
  }
  if { ![validuser $arg] } {
    putserv "PRIVMSG $chan :Error: Invalid handle."
    return
  }
  chattr $arg |+o $chan
  putserv "PRIVMSG $chan :Added $arg as operator on $chan."
}

bind pub - .remvoice pub_remvoice
proc pub_remvoice {nick host hand chan arg} {
  if { ![matchattr $hand o] && ![matchchanattr $hand o $chan] } {
    putserv "PRIVMSG $chan :Access denied."
    return
  }
  if { ![isloggedin $hand "$nick!$host"] } {
    putserv "PRIVMSG $chan :Access denied: not logged in."
    return
  }
  if { $arg == "" } {
    putserv "PRIVMSG $chan :Usage: remvoice <handle>"
    return
  }
  if { ![validuser $arg] } {
    putserv "PRIVMSG $chan :Error: Invalid handle."
    return
  }
  chattr $arg |-v $chan
  putserv "PRIVMSG $chan :Removed $arg as voice on $chan."
}

bind pub - .remop pub_remop
proc pub_remop {nick host hand chan arg} {
  if { ![matchattr $hand o] && ![matchchanattr $hand m $chan] } {
    putserv "PRIVMSG $chan :Access denied."
    return
  }
  if { ![isloggedin $hand "$nick!$host"] } {
    putserv "PRIVMSG $chan :Access denied: not logged in."
    return
  }
  if { $arg == "" } {
    putserv "PRIVMSG $chan :Usage: remop <handle>"
    return
  }
  if { ![validuser $arg] } {
    putserv "PRIVMSG $chan :Error: Invalid handle."
    return
  }
  chattr $arg |-o $chan
  putserv "PRIVMSG $chan :Removed $arg as operator on $chan."
}

bind pub - .addmaster pub_addmaster
proc pub_addmaster {nick host hand chan arg} {
  if { ![matchattr $hand m] && ![matchchanattr $hand n $chan] } {
    putserv "PRIVMSG $chan :Access denied."
    return
  }
  if { ![isloggedin $hand "$nick!$host"] } {
    putserv "PRIVMSG $chan :Access denied: not logged in."
    return
  }
  if { $arg == "" } {
    putserv "PRIVMSG $chan :Usage: addmaster <handle>"
    return
  }
  if { ![validuser $arg] } {
    putserv "PRIVMSG $chan :Error: Invalid handle."
    return
  }
  chattr $arg |+m $chan
  putserv "PRIVMSG $chan :Added $arg as master on $chan."
}

bind pub - .remmaster pub_remmaster
proc pub_remmaster {nick host hand chan arg} {
  if { ![matchattr $hand m] && ![matchchanattr $hand n $chan] } {
    putserv "PRIVMSG $chan :Access denied."
    return
  }
  if { ![isloggedin $hand "$nick!$host"] } {
    putserv "PRIVMSG $chan :Access denied: not logged in."
    return
  }
  if { $arg == "" } {
    putserv "PRIVMSG $chan :Usage: remmaster <handle>"
    return
  }
  if { ![validuser $arg] } {
    putserv "PRIVMSG $chan :Error: Invalid handle."
    return
  }
  chattr $arg |-m $chan
  putserv "PRIVMSG $chan :Removed $arg as master on $chan."
}

bind pub - .addowner pub_addowner
proc pub_addowner {nick host hand chan arg} {
  if { ![matchattr $hand m] && ![matchchanattr $hand n $chan] } {
    putserv "PRIVMSG $chan :Access denied."
    return
  }
  if { ![isloggedin $hand "$nick!$host"] } {
    putserv "PRIVMSG $chan :Access denied: not logged in."
    return
  }
  if { $arg == "" } {
    putserv "PRIVMSG $chan :Usage: addowner <handle>"
    return
  }
  if { ![validuser $arg] } {
    putserv "PRIVMSG $chan :Error: Invalid handle."
    return
  }
  chattr $arg |+n $chan
  putserv "PRIVMSG $chan :Added $arg as OWNER on $chan!"
}

bind pub - .remowner pub_remowner
proc pub_remowner {nick host hand chan arg} {
  if { ![matchattr $hand m] && ![matchchanattr $hand n $chan] } {
    putserv "PRIVMSG $chan :Access denied."
    return
  }
  if { ![isloggedin $hand "$nick!$host"] } {
    putserv "PRIVMSG $chan :Access denied: not logged in."
    return
  }
  if { $arg == "" } {
    putserv "PRIVMSG $chan :Usage: remowner <handle>"
    return
  }
  if { ![validuser $arg] } {
    putserv "PRIVMSG $chan :Error: Invalid handle."
    return
  }
  chattr $arg |-n $chan
  putserv "PRIVMSG $chan :Removed $arg as owner on $chan."
}

bind pub - .adduser pub_adduser
proc pub_adduser {nick host hand chan arg} {
  if { ![matchattr $hand m] && ![matchchanattr $hand m $chan] } {
    putserv "PRIVMSG $chan :Access denied."
    return
  }
  if { ![isloggedin $hand "$nick!$host"] } {
    putserv "PRIVMSG $chan :Access denied: not logged in."
    return
  }
  if { [llength $arg] != 2 } {
    putserv "PRIVMSG $chan :Usage: adduser <user> <mask>"
    return
  }
  adduser [lindex $arg 0] [lindex $arg 1]
  putserv "PRIVMSG $chan :Added user [lindex $arg 0] with hostmask [lindex $arg 1]."
}

bind pub - .deluser pub_deluser
proc pub_deluser {nick host hand chan arg} {
  if { ![matchattr $hand m] && ![matchchanattr $hand m $chan] } {
    putserv "PRIVMSG $chan :Access denied."
    return
  }
  if { ![isloggedin $hand "$nick!$host"] } {
    putserv "PRIVMSG $chan :Access denied: not logged in."
    return
  }
  if { [llength $arg] != 2 } {
    putserv "PRIVMSG $chan :Usage: deluser <user>"
    return
  }
  if { ![validuser $arg] } {
    putserv "PRIVMSG $chan :Error: Invalid handle."
    return
  }
  if { [matchattr $arg f] || [matchattr $arg o] || [matchattr $arg t] || [matchattr $arg j] || [matchattr $arg a] || [matchattr $arg q] || [matchattr $arg v] || [matchattr $arg k] || [matchattr $arg d] || [matchattr $arg c] } {
    putserv "PRIVMSG $chan :User has special global flags, refused to delete."
    return
  }
  foreach tgt [chanlist] {
    if { $tgt != $chan } {
      if { [matchchanattr $arg f $tgt] || [matchchanattr $arg o $tgt] || [matchchanattr $arg a $tgt] || [matchchanattr $arg q $tgt] || [matchchanattr $arg v $tgt] || [matchchanattr $arg k $tgt] || [matchchanattr $arg d $tgt] } {
        putserv "PRIVMSG $chan :User has special flags on $tgt, refused to delete."
        return
      }
    }
  }
  deluser $arg
  putserv "PRIVMSG $chan :Deleted user $arg."
}

bind pub - .bans pub_bans
proc pub_bans {nick host hand chan arg} {
  global timezone
  foreach tgt [banlist] {
    if { [lindex $tgt 2] != 0 } {
      if { [lindex $tgt 4] != 0 } {
        putserv "NOTICE $nick :Global ban: [lindex $tgt 0] (\[[lindex $tgt 1]\] Created by [lindex $tgt 5], expiring at [ctime [lindex $tgt 2]] $timezone, created at [ctime [lindex $tgt 3]] $timezone, last used at [ctime [lindex $tgt 4]] $timezone.)"
      }
      if { [lindex $tgt 4] == 0 } {
        putserv "NOTICE $nick :Global ban: [lindex $tgt 0] (\[[lindex $tgt 1]\] Created by [lindex $tgt 5], expiring at [ctime [lindex $tgt 2]] $timezone, created at [ctime [lindex $tgt 3]] $timezone, never used.)"
      }
    }
    if { [lindex $tgt 2] == 0 } {
      if { [lindex $tgt 4] != 0 } {
        putserv "NOTICE $nick :Global permanent ban: [lindex $tgt 0] (\[[lindex $tgt 1]\] Created by [lindex $tgt 5], created at [ctime [lindex $tgt 3]] $timezone, last used at [ctime [lindex $tgt 4]] $timezone.)"
      }
      if { [lindex $tgt 4] == 0 } {
        putserv "NOTICE $nick :Global permanent ban: [lindex $tgt 0] (\[[lindex $tgt 1]\] Created by [lindex $tgt 5], created at [ctime [lindex $tgt 3]] $timezone, never used.)"
      }
    }
  }
  foreach tgt [banlist $chan] {
    if { [lindex $tgt 2] != 0 } {
      if { [lindex $tgt 4] != 0 } {
        putserv "NOTICE $nick :Channel ban: [lindex $tgt 0] (\[[lindex $tgt 1]\] Created by [lindex $tgt 5], expiring at [ctime [lindex $tgt 2]] $timezone, created at [ctime [lindex $tgt 3]] $timezone, last used at [ctime [lindex $tgt 4]] $timezone.)"
      }
      if { [lindex $tgt 4] == 0 } {
        putserv "NOTICE $nick :Channel ban: [lindex $tgt 0] (\[[lindex $tgt 1]\] Created by [lindex $tgt 5], expiring at [ctime [lindex $tgt 2]] $timezone, created at [ctime [lindex $tgt 3]] $timezone, never used.)"
      }
    }
    if { [lindex $tgt 2] == 0 } {
      if { [lindex $tgt 4] != 0 } {
        putserv "NOTICE $nick :Channel permanent ban: [lindex $tgt 0] (\[[lindex $tgt 1]\] Created by [lindex $tgt 5], created at [ctime [lindex $tgt 3]] $timezone, last used at [ctime [lindex $tgt 4]] $timezone.)"
      }
      if { [lindex $tgt 4] == 0 } {
        putserv "NOTICE $nick :Channel permanent ban: [lindex $tgt 0] (\[[lindex $tgt 1]\] Created by [lindex $tgt 5], created at [ctime [lindex $tgt 3]] $timezone, never used.)"
      }
    }
  }
  putlog "($host) !$hand! $chan BANS"
}

bind pub o|o .+ban pub_addban
proc pub_addban {nick host hand chan arg} {
  if { ![isloggedin $hand "$nick!$host"] } {
    putserv "PRIVMSG $chan :Access denied: not logged in."
    return
  }
  if { [llength $arg] < 2 } {
    putserv "PRIVMSG $chan :Usage: +ban <banmask> <lifetime-in-minutes> \[reason\]"
    putserv "PRIVMSG $chan :Note that if the lifetime is set to 0, the ban is permanent."
    return
  }
  if { [lindex $arg 2] != "" } {
    set cmt [lreplace $arg 0 1]
  }
  if { [lindex $arg 2] == "" } {
    set cmt "Requested by $nick ($hand)"
  }
  set life [expr [lindex $arg 1]]
  newchanban $chan [lindex $arg 0] $hand $cmt $life
  if { $life == 0 } {
    putserv "PRIVMSG $chan :Added ban."
  }
  if { $life != 0 } {
    putserv "PRIVMSG $chan :Added ban for $life seconds"
  }
  putlog "($host) !$hand! $chan +BAN $arg"
}

bind pub o|o .-ban pub_delban
proc pub_delban {nick host hand chan arg} {
  if { ![isloggedin $hand "$nick!$host"] } {
    putserv "PRIVMSG $chan :Access denied: not logged in."
    return
  }
  if { [llength $arg] < 1 } {
    putserv "PRIVMSG $chan :Usage: -ban <banmask>"
    return
  }
  killchanban $chan [lindex $arg 0]
  putserv "PRIVMSG $chan :Removed ban."
  putlog "($host) !$hand! $chan -BAN $arg"
}

### END ###
putlog "Eggdrop Extensions v$eggext_version by Vulture loaded."
