# topiceme 1.0 by k-rist (k-rist@rsn.hk-r.se)
#
# This script remebers the latest topic. If the topic at any time is set to 
# nothing, the bot will restore the topic.
# This can happen if the bot disconnects, cycles or if a user sets the 
# topic to nothing.
#
# Intended users are eggdrop 1.3.2x bots.
# The script can handle multiple channels.
#
# Commands:
# topicme - enables or disables this feature, only available for 
#         - owners (n) via dcc
#
# The command can only be issued in dcc mode, and must be preceded with .
# For example: .topicme
#
# The command requires the owner flag (n) flag to be set on the user
# Note: Be careful when you select owners to your bot!
#
# Wishlist:
# Better errormessages
#
###############################################################################

# Enable this feature (script)? (0/1)
# 0 disables the feature, 1 enables the feature
set topicme 1

# Which prefix should the topic files have?
set topicmeprefix ".topicme"

# Which channels to monitor?
set topicmechannels "#ud"





##################################################
# You should NOT change anything below this line #
##################################################

set topicmeversion "topicme 1.0"

bind dcc n topicme dcc_topicme
bind topc - * topc_topicme
bind mode - "* +o" mode_topicme

proc dcc_topicme {handle idx arg} {
    global topicme
    global topicmeversion
    if {$topicme} {
	set topicme 0
	set answer "$topicmeversion disabled - topic will be not restored"
    } else {
	set topicme 1
	set answer "$topicmeversion enabled - topic will be restored"
    }
    putdcc $idx $answer
    return 1
}

proc topc_topicme {nick uhost handle channel topic} {
    global topicme
    global botnick
    if {$topicme && $nick != $botnick} {
	checktopic $channel $topic
    }
    return 1
}

proc mode_topicme {nick uhost handle channel modechange victim} {
    global topicme
    global botnick
    if {$topicme && $victim == $botnick} {
	set topic [topic $channel]
	checktopic $channel $topic
    }
    return 1
}

proc checktopic {channel topic} {
    global topicmeprefix
    global topicmeversion
    if {[checkchannel $channel]} {
	if {$topic == ""} {
	    set channelfile $topicmeprefix$channel
	    if {[checkfile $channelfile "r"]} {
		set fileio [open $channelfile "r"]
		set restoredtopic [gets $fileio]
		close $fileio
		if {$restoredtopic != ""} {
		    putserv "TOPIC $channel :$restoredtopic"
		    return 1
		}
	    }
	}
	newtopic $channel $topic
    }
    return 1
}

proc newtopic {channel topic} {
    global topicmeprefix
    global topicmeversion
    set channelfile $topicmeprefix$channel
    if {[file exists $channelfile]} {
	if {![checkfile $channelfile "w"]} {
	    putlog "$topicmeversion could not edit the file $channelfile"
	    return 0
	}
    }
    set fileio [open $channelfile "w"]
    puts $fileio $topic
    flush $fileio
    close $fileio
    return 1
}

# Checks if the channel is being monitored
# channel - the channel to check
# return 1 if ok, 0 otherwise
proc checkchannel {channel} {
    global topicmechannels
    if {$channel != ""} {
        foreach monitorchannel $topicmechannels {
            set monitorchannel [string tolower $monitorchannel]
            if {$channel == $monitorchannel} {
                return 1
            }
        }
    }
    return 0
}

# Checks if the file exists and is a file (and not a directory)
# file - the file to check
# rw   - set "r" if file should be checked for readability
#      - set "w" if file should be checked for writability
# return 1 if file is ok, 0 if it failed
proc checkfile {file rw} {
    if {[file exists $file] && [file isfile $file]} {
        if {$rw == "r"} {
            if {[file readable $file]} {
                return 1
            } else {
                return 0
            }
        } elseif {$rw == "w"} {
            if {[file writable $file]} {
                return 1
            } else {
                return 0
            }
        } else {
            return 1
        }
    }
    return 0
}

putlog "$topicmeversion loaded."
